# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0


import copy
# Migration of wms_cfg.py.  As at commit  c44c5e61c7fb9

# Reusable Chunks 1. Resource limit configurations

reslim_landsat = {
    "wms": {
        "zoomed_out_fill_colour": [150,180,200,160],
        "min_zoom_factor": 15.0,
        # "max_datasets": 16, # Defaults to no dataset limit
    },
    "wcs": {
        # "max_datasets": 16, # Defaults to no dataset limit
    }
}

# Reusable Chunks 2. Band lists.

bands_ls8 = {
    "red": [],
    "green": [],
    "blue": [ ],
    "nir": [ "near_infrared" ],
    "swir1": [ "shortwave_infrared_1", "near_shortwave_infrared" ],
    "swir2": [ "shortwave_infrared_2", "far_shortwave_infrared" ],
    "coastal_aerosol": [ ],
}

bands_ls = {
    "red": ['pink'],
    "green": [],
    "blue": ['azure' ],
    "nir": [ "near_infrared" ],
    "swir1": [ "shortwave_infrared_1", "near_shortwave_infrared" ],
    "swir2": [ "shortwave_infrared_2", "far_shortwave_infrared" ],
}

# Reusable Chunks 3. Styles

style_ls_simple_rgb = {
        "name": "simple_rgb",
        "title": "Simple RGB",
        "abstract": "Simple true-colour image, using the red, green and blue bands",
        "components": {
            "red": {
                "pink": 1.0
            },
            "green": {
                "green": 1.0
            },
            "blue": {
                "azure": 1.0
            }
        },
        "scale_range": [0.0, 3000.0]
}


style_ls_simple_rg = {
    "inherits": {
        "layer": "ls8_nbart_geomedian_annual",
        "style": "simple_rgb",
    },
    "name": "simple_rg",
        "title": "Simple RG",
        "abstract": "Simple red-green image, using the red, green and blue bands",
        "components": {
            "green": {
                "green": 0.6,
                "blue": 0.4
            },
            "blue": {
            }
        },
}

style_ls_irg = {
    "name": "infrared_green",
    "title": "False colour - Green, SWIR, NIR",
    "abstract": "False Colour image with SWIR1->Red, NIR->Green, and Green->Blue",
    "components": {
        "red": {
            "swir1": 1.0
        },
        "green": {
            "nir": 1.0
        },
        "blue": {
            "green": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}
style_ls_irr = {
    "inherits": style_ls_irg,
    "name": "infrared_red",
    "title": "False colour - Red, SWIR, NIR",
    "abstract": "False Colour image with SWIR1->Red, NIR->Green, and Red->Blue",
    "components": {
        "red": {
            "swir1": 1.0
        },
        "green": {
            "nir": 1.0
        },
        "blue": {
            "red": 1.0
        }
    },
}

style_ls_rgbndvi = {
    "name": "rgb_ndvi",
    "title": "NDVI plus RGB",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "component_ratio": 0.6,
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "nir",
            "band2": "red"
        }
    },
    "needed_bands": ["red", "nir"],
    "components": {
        "red": {
            "red": 1.0
        },
        "green": {
            "green": 1.0
        },
        "blue": {
            "blue": 1.0
        }
    },
    "scale_range": [0.0, 3000.0],
    "color_ramp": [
        {
            "value": -0.00000001,
            "color": "#8F3F20",
            "alpha": 0.0
        },
        {
            "value": 0.0,
            "color": "#8F3F20",
            "alpha": 1.0
        },
        {
            "value": 0.1,
            "color": "#A35F18"
        },
        {
            "value": 0.2,
            "color": "#B88512"
        },
        {
            "value": 0.3,
            "color": "#CEAC0E"
        },
        {
            "value": 0.4,
            "color": "#E5D609"
        },
        {
            "value": 0.5,
            "color": "#FFFF0C"
        },
        {
            "value": 0.6,
            "color": "#C3DE09"
        },
        {
            "value": 0.7,
            "color": "#88B808"
        },
        {
            "value": 0.8,
            "color": "#529400"
        },
        {
            "value": 0.9,
            "color": "#237100"
        },
        {
            "value": 1.0,
            "color": "#114D04"
        }
    ],
}

style_ls_ndvi = {
    "name": "ndvi",
    "title": "NDVI - Red, NIR",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "nir",
            "band2": "red"
        }
    },
    "needed_bands": ["red", "nir"],
    "color_ramp": [
        {
            "value": -0.00000001,
            "color": "#8F3F20",
            "alpha": 0.0
        },
        {
            "value": 0.0,
            "color": "#8F3F20",
            "alpha": 1.0
        },
        {
            "value": 0.1,
            "color": "#A35F18"
        },
        {
            "value": 0.2,
            "color": "#B88512"
        },
        {
            "value": 0.3,
            "color": "#CEAC0E"
        },
        {
            "value": 0.4,
            "color": "#E5D609"
        },
        {
            "value": 0.5,
            "color": "#FFFF0C"
        },
        {
            "value": 0.6,
            "color": "#C3DE09"
        },
        {
            "value": 0.7,
            "color": "#88B808"
        },
        {
            "value": 0.8,
            "color": "#529400"
        },
        {
            "value": 0.9,
            "color": "#237100"
        },
        {
            "value": 1.0,
            "color": "#114D04"
        }
    ],
    "legend": {
        "begin": "0.0",
        #  "end": "1.0",
        "ticks_every": "0.2",

        "units": "unitless",

        "title": "Vegetation Index - Red/NIR",

        "decimal_places": 2,

        "tick_labels": {
            "default": {
                "prefix": "+",
                "suffix": ")"
            },
            "0.0": {
                "prefix": "(",
                "label": "Dead Desert"
            },
            "0.2": {
                "label": "0.2",
                "suffix": ""
            },
            "0.8": {
                "prefix": ":",
                "label": "-"
            },
            "1.0": {
                "prefix": "(",
                "label": "Lush Jungle"
            }
        }
    },
    # Define behaviour(s) for multi-date requests. If not declared, style only supports single-date requests.
    "multi_date": [
        # A multi-date handler.  Different handlers can be declared for different numbers of dates in a request.
        {
            # The count range for which this handler is to be used - a tuple of two ints, the smallest and
            # largest date counts for which this handler will be used.  Required.
            "allowed_count_range": [2, 8],
            # A function, expressed in the standard format as described elsewhere in this example file.
            # The function is assumed to take one arguments, an xarray Dataset.
            # The function returns an xarray Dataset with a single band, which is the input to the
            # colour ramp defined below.
            "animate": True,
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_pass"
            },
            "color_ramp": [
                {
                    "value": -0.00000001,
                    "color": "#8F3F20",
                    "alpha": 0.0
                },
                {
                    "value": 0.0,
                    "color": "#8F3F20",
                    "alpha": 1.0
                },
                {
                    "value": 0.1,
                    "color": "#A35F18"
                },
                {
                    "value": 0.2,
                    "color": "#B88512"
                },
                {
                    "value": 0.3,
                    "color": "#CEAC0E"
                },
                {
                    "value": 0.4,
                    "color": "#E5D609"
                },
                {
                    "value": 0.5,
                    "color": "#FFFF0C"
                },
                {
                    "value": 0.6,
                    "color": "#C3DE09"
                },
                {
                    "value": 0.7,
                    "color": "#88B808"
                },
                {
                    "value": 0.8,
                    "color": "#529400"
                },
                {
                    "value": 0.9,
                    "color": "#237100"
                },
                {
                    "value": 1.0,
                    "color": "#114D04"
                }
            ],

            # The multi-date color ramp.  May be defined as an explicit colour ramp, as shown above for the single
            # date case; or may be defined with a range and unscaled color ramp as shown here.
            #
            # The range specifies the min and max values for the color ramp.  Required if an explicit color
            # ramp is not defined.
            # "range": [-1.0, 1.0],
            # The name of a named matplotlib color ramp.
            # Reference here: https://matplotlib.org/examples/color/colormaps_reference.html
            # Only used if an explicit colour ramp is not defined.  Optional - defaults to a simple (but
            # kind of ugly) blue-to-red rainbow ramp.
            # "mpl_ramp": "RdBu",
            # The feature info label for the multi-date index value.
            "feature_info_label": "nbr_delta",
            "legend": {
                "title": "Difference",
                "start": "-0.50",
                "end": "0.88",
                "ticks": ["-0.50", "-0.25", "-0.1", "0.27", "0.44", "0.66", "0.88"],
                "tick_labels": {
                    "-0.50": {"prefix": "<"},
                    "0.88": {"prefix": ">", "label": "1.30"},
                },
                "decimal_places": 2,
            }
        }
    ]
}

style_ls_ndvi_expr = {
"name": "ndvi_expr",
"title": "NDVI - Red, NIR",
"abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
"index_expression": "(nir-red)/(nir+red)",
"color_ramp": [
    {
        "value": -0.00000001,
        "color": "#8F3F20",
        "alpha": 0.0
    },
    {
        "value": 0.0,
            "color": "#8F3F20",
            "alpha": 1.0
        },
        {
            "value": 0.1,
            "color": "#A35F18"
        },
        {
            "value": 0.2,
            "color": "#B88512"
        },
        {
            "value": 0.3,
            "color": "#CEAC0E"
        },
        {
            "value": 0.4,
            "color": "#E5D609"
        },
        {
            "value": 0.5,
            "color": "#FFFF0C"
        },
        {
            "value": 0.6,
            "color": "#C3DE09"
        },
        {
            "value": 0.7,
            "color": "#88B808"
        },
        {
            "value": 0.8,
            "color": "#529400"
        },
        {
            "value": 0.9,
            "color": "#237100"
        },
        {
            "value": 1.0,
            "color": "#114D04"
        }
    ],
    "legend": {
       "begin": "0.0",
       #  "end": "1.0",
       "ticks_every": "0.2",

       "units": "unitless",

       "title": "Vegetation Index - Red/NIR",

       "decimal_places": 2,

       "tick_labels": {
           "default": {
               "prefix": "+",
               "suffix": ")"
           },
           "0.0": {
               "prefix": "(",
               "label": "Dead Desert"
           },
           "0.2": {
               "label": "0.2",
               "suffix": ""
           },
           "0.8": {
                "prefix": ":",
                "label": "-" 
           },
           "1.0": {
               "prefix": "(",
               "label": "Lush Jungle"
           }
        }
    },
    # Define behaviour(s) for multi-date requests. If not declared, style only supports single-date requests.
    "multi_date": [
        # A multi-date handler.  Different handlers can be declared for different numbers of dates in a request.
        {
            # The count range for which this handler is to be used - a tuple of two ints, the smallest and
            # largest date counts for which this handler will be used.  Required.
            "allowed_count_range": [2, 2],
            # A function, expressed in the standard format as described elsewhere in this example file.
            # The function is assumed to take one arguments, an xarray Dataset.
            # The function returns an xarray Dataset with a single band, which is the input to the
            # colour ramp defined below.
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_delta"
            },

            "color_ramp": [
                {
                    "value": -0.5,
                    "color": "#768642",
                    "alpha": 0.0
                },
                {
                    "value": -0.5,
                    "color": "#768642",
                },
                {
                    "value": -0.25,
                    "color": "#768642",
                    "alpha": 1.0,
                },
                {
                    "value": -0.25,
                    "color": "#a4bd5f"
                },
                {
                    "value": -0.1,
                    "color": "#a4bd5f",
                },
                {
                    "value": -0.1,
                    "color": "#00e05d"
                },
                {
                    "value": 0.1,
                    "color": "#00e05d"
                },
                {
                    "value": 0.1,
                    "color": "#fdf950",
                },
                {
                    "value": 0.27,
                    "color": "#fdf950",
                },
                {
                    "value": 0.27,
                    "color": "#ffae52"
                },
                {
                    "value": 0.44,
                    "color": "#ffae52",
                },
                {
                    "value": 0.44,
                    "color": "#ff662e"
                },
                {
                    "value": 0.66,
                    "color": "#ff662e",
                },
                {
                    "value": 0.66,
                    "color": "#ad28cc"
                },
                {
                    "value": 0.88,
                    "color": "#ad28cc",
                },
            ],
            # The multi-date color ramp.  May be defined as an explicit colour ramp, as shown above for the single
            # date case; or may be defined with a range and unscaled color ramp as shown here.
            #
            # The range specifies the min and max values for the color ramp.  Required if an explicit color
            # ramp is not defined.
            # "range": [-1.0, 1.0],
            # The name of a named matplotlib color ramp.
            # Reference here: https://matplotlib.org/examples/color/colormaps_reference.html
            # Only used if an explicit colour ramp is not defined.  Optional - defaults to a simple (but
            # kind of ugly) blue-to-red rainbow ramp.
            # "mpl_ramp": "RdBu",
            # The feature info label for the multi-date index value.
            "feature_info_label": "nbr_delta",
            "legend": {
                "title": "Difference",
                "start": "-0.50",
                "end": "0.88",
                "ticks": ["-0.50", "-0.25", "-0.1", "0.27", "0.44", "0.66", "0.88"],
                "tick_labels": {
                    "-0.50": {"prefix": "<"},
                    "0.88": {"prefix": ">", "label": "1.30"},
                },
                "decimal_places": 2,
            }
        }
    ]
}


style_ls8_nbr = {
    "name": "NBR",
    "title": "Normalised Burn Ratio",
    "abstract": "Normalised Burn Ratio - a derived index that that uses the differences in the way health green vegetation and burned vegetation reflect light to find burned area",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "nir",
            "band2": "swir2"
        }
    },
    "needed_bands": ["nir", "swir2"],
    "color_ramp": [
        {
            "value": -1.0,
            "color": "#67001F",
            "alpha": 0.0,
        },
        {
            "value": -1.0,
            "color": "#67001F",
        },
        {
            "value": -0.8,
            "color": "#B2182B",
        },
        {
            "value": -0.4,
            "color": "#D6604D"
        },
        {
            "value": -0.2,
            "color": "#F4A582"
        },
        {
            "value": -0.1,
            "color": "#FDDBC7"
        },
        {
            "value": 0,
            "color": "#F7F7F7",
        },
        {
            "value": 0.2,
            "color": "#D1E5F0"
        },
        {
            "value": 0.4,
            "color": "#92C5DE"
        },
        {
            "value": 0.6,
            "color": "#4393C3"
        },
        {
            "value": 0.9,
            "color": "#2166AC"
        },
        {
            "value": 1.0,
            "color": "#053061",
        }
    ],
    "legend": {
        "show_legend": True,
        "start": "-1.0",
        "end": "1.0",
        "ticks": [ "-1.0", "0.0", "1.0" ],
        "tick_labels": {
            "-1.0": { "prefix": "<"},
            "1.0": { "suffix": ">"}
        }
    },
    # Define behaviour(s) for multi-date requests. If not declared, style only supports single-date requests.
    "multi_date": [
        # A multi-date handler.  Different handlers can be declared for different numbers of dates in a request.
        {
            # The count range for which this handler is to be used - a tuple of two ints, the smallest and
            # largest date counts for which this handler will be used.  Required.
            "allowed_count_range": [2, 2],
            # A function, expressed in the standard format as described elsewhere in this example file.
            # The function is assumed to take one arguments, an xarray Dataset.
            # The function returns an xarray Dataset with a single band, which is the input to the
            # colour ramp defined below.
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_delta"
            },

           "color_ramp": [
                {
                    "value": -0.5,
                    "color": "#768642",
                    "alpha": 0.0
                },
                {
                    "value": -0.5,
                    "color": "#768642",
                },
                {
                    "value": -0.25,
                    "color": "#768642",
                    "alpha": 1.0,
                },
                {
                    "value": -0.25,
                    "color": "#a4bd5f"
                },
                {
                    "value": -0.1,
                    "color": "#a4bd5f",
                },
                {
                    "value": -0.1,
                    "color": "#00e05d"
                },
                {
                    "value": 0.1,
                    "color": "#00e05d"
                },
                {
                    "value": 0.1,
                    "color": "#fdf950",
                },
                {
                    "value": 0.27,
                    "color": "#fdf950",
                },
                {
                    "value": 0.27,
                    "color": "#ffae52"
                },
                {
                    "value": 0.44,
                    "color": "#ffae52",
                },
                {
                    "value": 0.44,
                    "color": "#ff662e"
                },
                {
                    "value": 0.66,
                    "color": "#ff662e",
                },
                {
                    "value": 0.66,
                    "color": "#ad28cc"
                },
                {
                    "value": 0.88,
                    "color": "#ad28cc",
                },
            ],
            # The multi-date color ramp.  May be defined as an explicit colour ramp, as shown above for the single
            # date case; or may be defined with a range and unscaled color ramp as shown here.
            #
            # The range specifies the min and max values for the color ramp.  Required if an explicit color
            # ramp is not defined.
            # "range": [-1.0, 1.0],
            # The name of a named matplotlib color ramp.
            # Reference here: https://matplotlib.org/examples/color/colormaps_reference.html
            # Only used if an explicit colour ramp is not defined.  Optional - defaults to a simple (but
            # kind of ugly) blue-to-red rainbow ramp.
            # "mpl_ramp": "RdBu",
            # The feature info label for the multi-date index value.
            "feature_info_label": "nbr_delta",
            "legend": {
                "title": "Difference",
                "start": "-0.50",
                "end": "0.88",
                "ticks": ["-0.50", "-0.25", "-0.1", "0.27", "0.44", "0.66", "0.88"],
                "tick_labels": {
                    "-0.50": {"prefix": "<"},
                    "0.88": {"prefix": ">", "label": "1.30"},
                },
                "decimal_places": 2,
            }
        }
    ]
}

style_ls_ndvi_alt1 = {
    "name": "ndvi-alt1",
    "title": "NDVI - Red, NIR (alt1)",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "near_infrared",
            "band2": "steve"
        }
    },
    "band_map": {
        "steve": "red"
    },
    "needed_bands": ["red", "near_infrared"],
    "range": [0.0, 1.0],
    "mpl_ramp": "RdBu"
}

style_ls_ndvi_delta = {
    "name": "ndvi-delta",
    "title": "NDVI - Red, NIR (delta)",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "nir",
            "band2": "red"
        }
    },
    "needed_bands": ["red", "nir"],
    "range": [0.0, 1.0],
    "mpl_ramp": "RdYlGn",
    "multi_date": [
        {
            "allowed_count_range": [2, 2],
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_delta",
            },
            "range": [-0.25, 0.25],
            "mpl_ramp": "RdBu",
            "legend": {
                "start": "-1.0",
                "end": "1.0",
                "title": "NDVI Difference",
            }
        }
    ]
}



style_ls_ndvi_alt2 = {
    "name": "ndvi-alt2",
    "title": "NDVI - Red, NIR (alt2)",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "nir",
            "band2": "red"
        }
    },
    "needed_bands": ["red", "nir"],
    "range": [0.0, 1.0],
    "mpl_ramp": "winter"
}

style_ls_ndvi_alt3 = {
    "name": "ndvi-alt3",
    "title": "NDVI - Red, NIR (alt2)",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "nir",
            "band2": "red"
        }
    },
    "needed_bands": ["red", "nir"],
    "range": [0.0, 1.0],
    "mpl_ramp": "plasma"
}


style_ls_ndwi = {
    "name": "ndwi",
    "title": "NDWI - Green, NIR",
    "abstract": "Normalised Difference Water Index - a derived index that correlates well with the existence of water (McFeeters 1996)",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "green",
            "band2": "nir"
        }
    },
    "needed_bands": ["green", "nir"],
    "color_ramp": [
        {
            "value": -0.1,
            "color": "#f7fbff",
            "alpha": 0.0
        },
        {
            "value": 0.0,
            "color": "#d8e7f5",
            "legend": {
                "prefix": "<"
            }
        },
        {
            "value": 0.1,
            "color": "#b0d2e8"
        },
        {
            "value": 0.2,
            "color": "#73b3d8",
            "legend": { }
        },
        {
            "value": 0.3,
            "color": "#3e8ec4"
        },
        {
            "value": 0.4,
            "color": "#1563aa",
            "legend": { }
        },
        {
            "value": 0.5,
            "color": "#08306b",
            "legend": {
                "prefix": ">"
            }
        }
    ],
    "legend": {
        "start": "0.0",
        "end": "0.5",
        "ticks": ["0.0", "0.2", "0.4", "0.5"],
        "tick_labels": {
            "0.0": {"prefix": "<", "label": "0"},
            "0.5": {"prefix": ">"},
        }
    }
}

style_ls_mndwi = {
    "name": "mndwi",
    "title": "MNDWI - Green, SWIR",
    "abstract": "Modified Normalised Difference Water Index - a derived index that correlates well with the existence of water (Xu 2006)",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {
            "band1": "green",
            "band2": "swir1"
        }
    },
    "needed_bands": ["green", "swir1"],
    "color_ramp": [
        {
            "value": -0.1,
            "color": "#f7fbff",
            "alpha": 0.0
        },
        {
            "value": 0.0,
            "color": "#d8e7f5"
        },
        {
            "value": 0.2,
            "color": "#b0d2e8"
        },
        {
            "value": 0.4,
            "color": "#73b3d8"
        },
        {
            "value": 0.6,
            "color": "#3e8ec4"
        },
        {
            "value": 0.8,
            "color": "#1563aa"
        },
        {
            "value": 1.0,
            "color": "#08306b"
        }
    ]
}

style_ls_pure_blue = {
    "name": "blue",
    "title": "Blue - 480",
    "abstract": "Blue band, centered on 480nm",
    "components": {
        "red": {
            "blue": 1.0
        },
        "green": {
            "blue": 1.0
        },
        "blue": {
            "blue": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_sentinel_pure_blue = {
    "name": "blue",
    "title": "Blue - 490",
    "abstract": "Blue band, centered on 490nm",
    "components": {
        "red": {
            "blue": 1.0
        },
        "green": {
            "blue": 1.0
        },
        "blue": {
            "blue": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_ls_pure_green = {
    "name": "green",
    "title": "Green - 560",
    "abstract": "Green band, centered on 560nm",
    "components": {
        "red": {
            "green": 1.0
        },
        "green": {
            "green": 1.0
        },
        "blue": {
            "green": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_ls_pure_red = {
    "name": "red",
    "title": "Red - 660",
    "abstract": "Red band, centered on 660nm",
    "components": {
        "red": {
            "red": 1.0
        },
        "green": {
            "red": 1.0
        },
        "blue": {
            "red": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_ls_pure_nir = {
    "name": "nir",
    "title": "Near Infrared (NIR) - 840",
    "abstract": "Near infra-red band, centered on 840nm",
    "components": {
        "red": {
            "nir": 1.0
        },
        "green": {
            "nir": 1.0
        },
        "blue": {
            "nir": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_sentinel_pure_nir = {
    "name": "nir",
    "title": "Near Infrared (NIR) - 870",
    "abstract": "Near infra-red band, centered on 870nm",
    "components": {
        "red": {
            "nir": 1.0
        },
        "green": {
            "nir": 1.0
        },
        "blue": {
            "nir": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_ls_pure_swir1 = {
    "name": "swir1",
    "title": "Shortwave Infrared (SWIR) - 1650",
    "abstract": "Short wave infra-red band 1, centered on 1650nm",
    "components": {
        "red": {
            "swir1": 1.0
        },
        "green": {
            "swir1": 1.0
        },
        "blue": {
            "swir1": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_sentinel_pure_swir1 = {
    "name": "swir1",
    "title": "Shortwave Infrared (SWIR) - 1610",
    "abstract": "Short wave infra-red band 1, centered on 1610nm",
    "components": {
        "red": {
            "swir1": 1.0
        },
        "green": {
            "swir1": 1.0
        },
        "blue": {
            "swir1": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}

style_ls_pure_swir2 = {
    "name": "swir2",
    "title": "Shortwave Infrared (SWIR) - 2220",
    "abstract": "Short wave infra-red band 2, centered on 2220nm",
    "components": {
        "red": {
            "swir2": 1.0
        },
        "green": {
            "swir2": 1.0
        },
        "blue": {
            "swir2": 1.0
        }
    },
    "scale_range": [0.0, 3000.0]
}


layer_geomedian_ls8 = {
                    "title": "Surface Reflectance 25m Annual Geomedian (Landsat 8)",
                    "name": "ls8_nbart_geomedian_annual",
                    "abstract": """
Data is only visible at higher resolutions; when zoomed-out the available area will be displayed
as a shaded region. The surface reflectance geometric median (geomedian) is a pixel composite
mosaic of a time series of earth observations. The value of a pixel in a an annual geomedian
image is the statistical median of all observations for that pixel from a calendar year.
Annual mosaics are available for the following years:

Landsat 8: 2013 to 2017;

For more information, see http://pid.geoscience.gov.au/dataset/ga/120374

For service status information, see https://status.dea.ga.gov.au
                    """,
                    "product_name": "ls8_nbart_geomedian_annual",
                    "bands": bands_ls,
                    "resource_limits": reslim_landsat,
                    "time_resolution": "year",
                    "image_processing": {
                        "extent_mask_func": "datacube_ows.ogc_utils.mask_by_val",
                        "always_fetch_bands": [],
                    },
                    "user_band_math": True,
                    "wcs": {
                        "native_resolution": [25.0, -25.0],
                        "default_bands": ["red", "green", "blue"],
                        "native_crs": "EPSG:3577"
                    },
                    "styling": {
                        "default_style": "simple_rgb",
                        "styles": [
                            style_ls_simple_rgb,
                            style_ls_simple_rg,
                            style_ls_irg,
                            style_ls_irr,
                            style_ls_rgbndvi,
                            style_ls_ndvi,
                            style_ls_ndvi_expr,
                            style_ls8_nbr,
                            style_ls_ndvi_alt1, style_ls_ndvi_alt2, style_ls_ndvi_alt3, 
                            style_ls_ndvi_delta,
                            style_ls_ndwi, style_ls_mndwi,
                            style_ls_pure_blue, style_ls_pure_green, style_ls_pure_red,
                            style_sentinel_pure_nir, style_sentinel_pure_swir1, style_ls_pure_swir2,
                        ]
                    }
                }
layer_geomedian_ls8_ltd = {
                    "inherits": {"layer": "ls8_nbart_geomedian_annual"},
                    "title": "Surface Reflectance 25m Annual Geomedian (Landsat 8) (limit to Barest Earth)",
                    "name": "ls8_nbart_geomedian_annual_limited",
                    "low_res_product_name": "ls8_fc_albers",
                    "resource_limits": {
                        "wms": {
                            "min_zoom_factor": 1800.0,
                        },
                    }
                }

layer_geomedian_ls7 = {
                    "title": "Surface Reflectance 25m Annual Geomedian (Landsat 7)",
                    "name": "ls7_nbart_geomedian_annual",
                    "abstract": """
Data is only visible at higher resolutions; when zoomed-out the available area will be displayed
as a shaded region. The surface reflectance geometric median (geomedian) is a pixel composite
mosaic of a time series of earth observations. The value of a pixel in a an annual geomedian
image is the statistical median of all observations for that pixel from a calendar year.
Annual mosaics are available for the following years:

Landsat 7: 2000 to 2017;

For more information, see http://pid.geoscience.gov.au/dataset/ga/120374

For service status information, see https://status.dea.ga.gov.au
                    """,
                    "product_name": "ls7_nbart_geomedian_annual",
                    "bands": bands_ls,
                    "resource_limits": reslim_landsat,
                    "image_processing": {
                        "extent_mask_func": "datacube_ows.ogc_utils.mask_by_val",
                        "always_fetch_bands": [],
                    },
                    "wcs": {
                        "native_resolution": [25.0, -25.0],
                        "default_bands": ["red", "green", "blue"],
                        "native_crs": "EPSG:3577"
                    },
                    "styling": {
                        "default_style": "simple_rgb",
                        "styles": [
                            style_ls_simple_rgb,
                            style_ls_irg, style_ls_irr, style_ls_ndvi, style_ls_ndvi_expr, style_ls_ndwi, style_ls_mndwi,
                            style_ls_ndvi_alt1, style_ls_ndvi_alt2, style_ls_ndvi_alt3, 
                            style_ls_ndvi_delta,
                            style_sentinel_pure_blue, style_ls_pure_green, style_ls_pure_red,
                            style_ls_pure_nir, style_ls_pure_swir1, style_ls_pure_swir2,
                        ]
                    }
                }

layer_geomedian_ls5 = {
                    "title": "Surface Reflectance 25m Annual Geomedian (Landsat 5)",
                    "name": "ls5_nbart_geomedian_annual",
                    "abstract": """
Data is only visible at higher resolutions; when zoomed-out the available area will be displayed
as a shaded region. The surface reflectance geometric median (geomedian) is a pixel composite
mosaic of a time series of earth observations. The value of a pixel in a an annual geomedian
image is the statistical median of all observations for that pixel from a calendar year.
Annual mosaics are available for the following years:

Landsat 5: 1988 to 1999, 2004 to 2007, 2009 to 2011;

For more information, see http://pid.geoscience.gov.au/dataset/ga/120374

For service status information, see https://status.dea.ga.gov.au
                    """,
                    "product_name": "ls5_nbart_geomedian_annual",
                    "bands": bands_ls,
                    "resource_limits": reslim_landsat,
                    "image_processing": {
                        "extent_mask_func": "datacube_ows.ogc_utils.mask_by_val",
                        "always_fetch_bands": [],
                    },
                    "wcs": {
                        "native_resolution": [25.0, -25.0],
                        "default_bands": ["red", "green", "blue"],
                        "native_crs": "EPSG:3577"
                    },
                    "styling": {
                        "default_style": "simple_rgb",
                        "styles": [
                            style_ls_simple_rgb,
                            style_ls_irg, style_ls_irr, style_ls_ndvi, style_ls_ndwi, style_ls_mndwi,
                            style_ls_ndvi_alt1, style_ls_ndvi_alt2, style_ls_ndvi_alt3, 
                            style_sentinel_pure_blue, style_ls_pure_green, style_ls_pure_red,
                            style_ls_pure_nir, style_ls_pure_swir1, style_ls_pure_swir2,
                        ]
                    }
                }
layer_barest_earth = {
            "title": "Landsat-8 Barest Earth",
            "abstract": """
A `weighted geometric median’ approach has been used to estimate the median surface reflectance of the barest state (i.e., least vegetation) observed through Landsat-8 OLI observations from 2013 to September 2018 to generate a six-band Landsat-8 Barest Earth pixel composite mosaic over the Australian continent.

The bands include BLUE (0.452 - 0.512), GREEN (0.533 - 0.590), RED, (0.636 - 0.673) NIR (0.851 - 0.879), SWIR1 (1.566 - 1.651) and SWIR2 (2.107 - 2.294) wavelength regions. The weighted median approach is robust to outliers (such as cloud, shadows, saturation, corrupted pixels) and also maintains the relationship between all the spectral wavelengths in the spectra observed through time. The product reduces the influence of vegetation and allows for more direct mapping of soil and rock mineralogy.

Reference: Dale Roberts, John Wilford, and Omar Ghattas (2018). Revealing the Australian Continent at its Barest, submitted.

Mosaics are available for the following years:
    Landsat 8: 2013 to 2017;
            """,
            "layers": [
                {
                    "title": "Landsat-8 Barest Earth 25m albers (Landsat-8)",
                    "name": "ls8_barest_earth_mosaic",
                    "abstract": """
A `weighted geometric median’ approach has been used to estimate the median surface reflectance of the barest state (i.e., least vegetation) observed through Landsat-8 OLI observations from 2013 to September 2018 to generate a six-band Landsat-8 Barest Earth pixel composite mosaic over the Australian continent.

The bands include BLUE (0.452 - 0.512), GREEN (0.533 - 0.590), RED, (0.636 - 0.673) NIR (0.851 - 0.879), SWIR1 (1.566 - 1.651) and SWIR2 (2.107 - 2.294) wavelength regions. The weighted median approach is robust to outliers (such as cloud, shadows, saturation, corrupted pixels) and also maintains the relationship between all the spectral wavelengths in the spectra observed through time. The product reduces the influence of vegetation and allows for more direct mapping of soil and rock mineralogy.

Reference: Dale Roberts, John Wilford, and Omar Ghattas (2018). Revealing the Australian Continent at its Barest, submitted.

Mosaics are available for the following years:
    Landsat 8: 2013 to 2017;

For service status information, see https://status.dea.ga.gov.au
                    """,
                    "product_name": "ls8_barest_earth_albers",
                    "bands": bands_ls,
                    "resource_limits": reslim_landsat,
                    "time_resolution": "year",
                    "image_processing": {
                        "extent_mask_func": "datacube_ows.ogc_utils.mask_by_val",
                        "always_fetch_bands": [],
                    },
                    "wcs": {
                        "native_crs": "EPSG:3577",
                        "native_resolution": [25, -25],
                        "default_bands": ["red", "green", "blue"]
                    },
                    "styling": {
                        "default_style": "simple_rgb",
                        "styles": [
                            style_ls_simple_rgb,
                            style_ls_irg, style_ls_irr, style_ls_ndvi,
                            style_ls_pure_blue, style_ls_pure_green, style_ls_pure_red,
                            style_sentinel_pure_nir, style_sentinel_pure_swir1, style_ls_pure_swir2,
                        ]
                    }

                }
            ]
        }

layer_folder_sr = {
            "title": "Surface Reflectance",
            "abstract": "",
            "layers": [
                layer_geomedian_ls8,
                layer_geomedian_ls8_ltd,
                layer_geomedian_ls7,
                layer_geomedian_ls5,
                layer_barest_earth,
            ]
        }



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0

try:
    from ._version import version as __version__
except ImportError:
    # Default version number.
    # Will only be used when running datacube-ows direct from source code (not properly installed)
    __version__ = "1.9.1?"



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0


import os

if os.environ.get("DATACUBE_OWS_CFG", "").startswith("local"):
    cfgbase = "local_cfg"
    trans_dir = "."
else:
    cfgbase = "config"
    trans_dir = "/code"

# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2021 OWS Contributors
# SPDX-License-Identifier: Apache-2.0
import copy
# Migration of wms_cfg.py.  As at commit  c44c5e61c7fb9

# Reusable Chunks 1. Resource limit configurations

# Actual Configuration

reslim_c3_ls = {
    "wms": {
        "zoomed_out_fill_colour": [150, 180, 200, 160],
        "min_zoom_factor": 15.0,
        # "max_datasets": 16, # Defaults to no dataset limit
    },
    "wcs": {
        # "max_datasets": 16, # Defaults to no dataset limit
    },
}

bands_c3_ls_8 = {
    "nbart_blue": ["nbart_blue"],
    "nbart_green": ["nbart_green"],
    "nbart_red": ["nbart_red"],
    "nbart_nir": ["nbart_nir", "nbart_near_infrared"],
    "nbart_swir_1": ["nbart_swir_1", "nbart_shortwave_infrared_1"],
    "nbart_swir_2": ["nbart_swir_2", "nbart_shortwave_infrared_2"],
    "oa_nbart_contiguity": ["oa_nbart_contiguity", "nbart_contiguity"],
    "oa_fmask": ["oa_fmask", "fmask"],
    "nbart_panchromatic": [],
    "nbart_coastal_aerosol": ["coastal_aerosol",  "nbart_coastal_aerosol"],
}

style_c3_fmask = {
    "name": "fmask",
    "title": "FMASK visualisation",
    "abstract": "Visualisation of fmask classification",
    "value_map": {
        "oa_fmask": [
            {
                "color": "#0000FF", # blue
                "title": "Water",
                "values": [5],
            },
            {
                "color": "#9090FF", # pale-blue
                "title": "Snow",
                "values": [4],
            },
            {
                "color": "#400000", # dark red
                "title": "Shadow",
                "values": [3],
            },
            {
                "color": "#E0FFF0", # pale green
                "title": "Cloud",
                "values": [2],
            },
            {
                "color": "#00C000", # bright green
                "title": "Valid",
                "values": [1],
            },
            {
                "color": "#000000",
                "title": "No Data",
                "values": [0],
            },
        ],
    },
    "legend": {
        "title": "FMASK categories",
        "ncols": 2
    }
}

style_c3_ndvi = {
    "name": "ndvi",
    "title": "NDVI - Red, NIR",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {"band1": "nbart_nir", "band2": "nbart_red"},
    },
    "needed_bands": ["nbart_red", "nbart_nir"],
    "color_ramp": [
        {"value": -0.0, "color": "#8F3F20", "alpha": 0.0},
        {"value": 0.0, "color": "#8F3F20", "alpha": 1.0},
        {"value": 0.1, "color": "#A35F18"},
        {"value": 0.2, "color": "#B88512"},
        {"value": 0.3, "color": "#CEAC0E"},
        {"value": 0.4, "color": "#E5D609"},
        {"value": 0.5, "color": "#FFFF0C"},
        {"value": 0.6, "color": "#C3DE09"},
        {"value": 0.7, "color": "#88B808"},
        {"value": 0.8, "color": "#529400"},
        {"value": 0.9, "color": "#237100"},
        {"value": 1.0, "color": "#114D04"},
    ],
    "pq_masks": [
        {
            "band": "oa_fmask",
            "values": [1,4,5],
            "invert": True,
        },
    ],
    "legend": {
        "begin": "0.0",
        "end": "1.0",
        "ticks_every": "0.2",
        "tick_labels": {
            "default": {
                "prefix": "+"
            },
            "0.0": {
                "prefix": "a ",
                "label": "bit"
            },
            "0.2": {
                "prefix": "d",
                "label": "000.2"
            },
            "0.8": {
                "prefix": "go"
            }
        },
        "title": "Nominal Inductance",
        "units": "microHenrys"
    }
# Define behaviour(s) for multi-date requests. If not declared, style only supports single-date requests.
}


style_c3_simple_rgb = {
    "name": "simple_rgb",
    "title": "Simple RGB",
    "abstract": "Simple true-colour image, using the red, green and blue bands",
    "components": {"red": {"nbart_red": 1.0}, "green": {"nbart_green": 1.0}, "blue": {"nbart_blue": 1.0}},
    "scale_range": [0.0, 3000.0],
    "pq_masks": [
        {
            "band": "oa_fmask",
            "values": [0],
            "invert": True,
        },
    ],
    "legend": {
         "show_legend": True,
         "url": "https://upload.wikimedia.org/wikipedia/commons/thumb/3/3d/Color_wheel_circle.png/440px-Color_wheel_circle.png"
    },
    "multi_date": [
        # A multi-date handler.  Different handlers can be declared for different numbers of dates in a request.
        {
            "preserve_user_date_order": True,
            # The count range for which this handler is to be used - a tuple of two ints, the smallest and
            # largest date counts for which this handler will be used.  Required.
            "allowed_count_range": [2, 12],
            "animate": True,
            "frame_duration": 1000,
            "legend": {
                "show_legend": True,
                "url": {
                    "en": "https://upload.wikimedia.org/wikipedia/commons/thumb/2/26/The_British_Empire.png/720px-The_British_Empire.png",
                    "fr": "https://upload.wikimedia.org/wikipedia/commons/thumb/8/88/France_cities.png/480px-France_cities.png"
                }
            }
        }
    ],
}


dea_c3_ls8_ard = {
    "title": "DEA C3 Landsat 8 ARD",
    "abstract": """
This product takes Landsat 8 imagery captured over the Australian continent and corrects for inconsistencies across land and coastal fringes. The result is accurate and standardised surface reflectance data, which is instrumental in identifying and quantifying environmental change.
The imagery is captured using the Operational Land Imager (OLI) and Thermal Infra-Red Scanner (TIRS) sensors aboard Landsat 8.
This product is a single, cohesive Analysis Ready Data (ARD) package, which allows you to analyse surface reflectance data as is, without the need to apply additional corrections.
It contains three sub-products that provide corrections or attribution information:
Surface Reflectance NBAR 3 (Landsat 8 OLI-TIRS)
Surface Reflectance NBART 3 (Landsat 8 OLI-TIRS)
Surface Reflectance OA 3 (Landsat 8 OLI-TIRS)
The resolution is a 30 m grid based on the USGS Landsat Collection 1 archive.""",
    # The WMS name for the layer
    "name": "ga_ls8c_ard_3",
    # The Datacube name for the associated data product
    "product_name": "ga_ls8c_ard_3",
    "bands": bands_c3_ls_8,
    "resource_limits": reslim_c3_ls,
    "default_time": "2020-01-01",
    "image_processing": {
        "extent_mask_func": "datacube_ows.ogc_utils.mask_by_val",
        "always_fetch_bands": [],
        "manual_merge": False,
    },
    "flags": [
        {
            "band": "oa_nbart_contiguity",
            "product": "ga_ls8c_ard_3",
            "ignore_time": False,
            "ignore_info_flags": [],
        },
        {
            "band": "oa_fmask",
            "product": "ga_ls8c_ard_3",
            "ignore_time": False,
            "ignore_info_flags": [],
        },
    ],
    "native_crs": "EPSG:3577",
    "native_resolution": [25, -25],
    "styling": {
        "default_style": "ndvi",
        "styles": [style_c3_ndvi, style_c3_fmask, style_c3_simple_rgb]
    }
}

ows_cfg = {
    "global": {
        # Master config for all services and products.
        "response_headers": {
            "Access-Control-Allow-Origin": "*",  # CORS header
        },
        "services": {
            "wms": True,
            "wcs": True,
            "wmts": True,
        },
        "published_CRSs": {
            "EPSG:3857": {  # Web Mercator
                "geographic": False,
                "horizontal_coord": "x",
                "vertical_coord": "y",
            },
            "EPSG:4326": {  # WGS-84
                "geographic": True,
                "vertical_coord_first": True
            },
            "EPSG:3577": {  # GDA-94, internal representation
                "geographic": False,
                "horizontal_coord": "x",
                "vertical_coord": "y",
            },
            "EPSG:3111": {  # VicGrid94 for delwp.vic.gov.au
                "geographic": False,
                "horizontal_coord": "x",
                "vertical_coord": "y",
            },
        },
        "allowed_urls": [
                "http://localhost:8000",
                "http://ows-configrefactor.dev.dea.ga.gov.au",
                "https://ows.services.dea.ga.gov.au",
                "https://ows.services.dev.dea.ga.gov.au",
                "https://ows.dev.dea.ga.gov.au",
                "https://ows.dea.ga.gov.au",
                "https://ows.services.devkube.dea.ga.gov.au",
                "https://nrt.services.dea.ga.gov.au",
                "https://geomedian.services.dea.ga.gov.au",
                "https://geomedianau.dea.ga.gov.au",
                "https://geomedian.dea.ga.gov.au",
                "https://nrt.dea.ga.gov.au",
                "https://nrt-au.dea.ga.gov.au"],


        "message_file": f"{trans_dir}/test_messages.po",
        "translations_directory": f"{trans_dir}/test_translations",
        "supported_languages": [
            "en", # Default
            "fr",
            "de",
            "sw",
        ],
        # Metadata to go straight into GetCapabilities documents
        "title": "Paul's Earth Australia - OGC Web Services",
        "abstract": "Digital Earth Australia OGC Web Services",
        "info_url": "dea.ga.gov.au/",
        "keywords": [
            "geomedian",
            "WOfS",
            "mangrove",
            "bare-earth",
            "NIDEM",
            "HLTC",
            "landsat",
            "australia",
            "time-series",
            "fractional-cover"
        ],
        "contact_info": {
            "person": "Digital Earth Australia",
            "organisation": "Geoscience Australia",
            "position": "",
            "address": {
                "type": "postal",
                "address": "GPO Box 378",
                "city": "Canberra",
                "state": "ACT",
                "postcode": "2609",
                "country": "Australia",
            },
            "telephone": "+61 2 6249 9111",
            "fax": "",
            "email": "earth.observation@ga.gov.au",
        },
        "fees": "",
        "access_constraints": "Copyright Commonwealth of Australia Geoscience Australia:2018",
        "use_extent_views": True,
    }, # END OF global SECTION
    "wms": {
        # Config for WMS service, for all products/layers
        "s3_url": "https://data.dea.ga.gov.au",
        "s3_bucket": "dea-public-data",
        "s3_aws_zone": "ap-southeast-2",

        "max_width": 512,
        "max_height": 512,
    }, # END OF wms SECTION
    "wcs": {
        # Config for WCS service, for all products/coverages
        "default_geographic_CRS": "EPSG:4326",
        "formats": {
            "GeoTIFF": {
                "renderers": {
                    "1": "datacube_ows.wcs1_utils.get_tiff",
                    "2": "datacube_ows.wcs2_utils.get_tiff",
                },
#               "renderer": "datacube_ows.wcs_utils.get_tiff",
                "mime": "image/geotiff",
                "extension": "tif",
                "multi-time": False
            },
            "netCDF": {
                "renderers": {
                    "1": "datacube_ows.wcs1_utils.get_netcdf",
                    "2": "datacube_ows.wcs2_utils.get_netcdf",
                },
                # "renderer": "datacube_ows.wcs_utils.get_netcdf",
                "mime": "application/x-netcdf",
                "extension": "nc",
                "multi-time": True,
            },
        },
        "native_format": "GeoTIFF",
    }, # END OF wcs SECTION
    "layers": [
        dea_c3_ls8_ard,
    ] # End of Layers List
} # End of ows_cfg object



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0

from .api import ows_index, AbortRun, CoordRange, LayerSignature, LayerExtent


__all__ = ["ows_index", "AbortRun", "CoordRange", "LayerSignature", "LayerExtent"]



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0

from threading import Lock
from typing import Optional

from datacube.drivers.driver_cache import load_drivers


TYPE_CHECKING = False
if TYPE_CHECKING:
    from datacube_ows.index.api import OWSAbstractIndexDriver

cache_lock = Lock()


class OWSIndexDriverCache:
    _instance = None
    _initialised = False
    def __new__(cls, *args, **kwargs) -> "OWSIndexDriverCache":
        if cls._instance is None:
            with cache_lock:
                if cls._instance is None:
                    cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, group: str) -> None:
        with cache_lock:
            if not self._initialised:
                self._initialised = True
                self._drivers = load_drivers(group)
    def __call__(self, name: str) -> Optional["OWSAbstractIndexDriver"]:
        """
        :returns: None if driver with a given name is not found

        :param name: Driver name
        :return: Returns IndexDriver
        """
        return self._drivers.get(name, None)

    def drivers(self) -> list[str]:
        """ Returns list of driver names
        """
        return list(self._drivers.keys())


def ows_index_drivers() -> list[str]:
    return OWSIndexDriverCache("datacube_ows.plugins.index").drivers()


def ows_index_driver_by_name(name: str) -> Optional["OWSAbstractIndexDriver"]:
    return OWSIndexDriverCache("datacube_ows.plugins.index")(name)



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0


from .layers import dea_c3_ls_ard, dea_c3_ls5_ard, dea_c3_ls7_ard, dea_c3_ls8_ard, s2_wofs_3

folder_c3_landsat = {
        "title": "Collection 3 Landsat Surface Relectance",
        "abstract": """
        """,
        "layers": [dea_c3_ls_ard, dea_c3_ls5_ard, dea_c3_ls7_ard, dea_c3_ls8_ard],
}

folder_c3_s2 = {
        "title": "Collection 3 Sentinel-2 Derived Products",
        "abstract": """
        """,
        "layers": [s2_wofs_3],
}




# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0

import click
import datetime

from threading import Lock
from typing import Any
from typing_extensions import override
from collections.abc import Iterable
from uuid import UUID
from sqlalchemy import text

from odc.geo import Geometry, CRS
from datacube import Datacube
from datacube.model import Product, Dataset, Range
from antimeridian import fix_shape

from datacube_ows.ows_configuration import OWSNamedLayer
from datacube_ows.index.api import OWSAbstractIndex, OWSAbstractIndexDriver, LayerSignature, LayerExtent, TimeSearchTerm
from datacube_ows.index.sql import run_sql
from .product_ranges import create_range_entry as create_range_entry_impl, get_ranges as get_ranges_impl
from ...utils import default_to_utc


class OWSPostgisIndex(OWSAbstractIndex):
    name: str = "postgis"

    # method to check database access (for ping op)
    @override
    def check_db_access(self, dc: Datacube) -> bool:
        db_ok = False
        try:
            with dc.index._db._give_me_a_connection() as conn:  # type: ignore[attr-defined]
                results = conn.execute(text("""
                    SELECT *
                    FROM ows.layer_ranges
                    LIMIT 1""")
                )
                for r in results:
                    db_ok = True
        except:
            pass
        return db_ok

    # method to delete obsolete schemas etc.
    @override
    def cleanup_schema(self, dc: Datacube):
        # No obsolete schema for postgis databases to clean up.
        pass

    # Schema creation method
    @override
    def create_schema(self, dc: Datacube) -> None:
        click.echo("Creating/updating schema and tables...")
        self._run_sql(dc, "ows_schema/create")

    # Permission management method
    @override
    def grant_perms(self, dc: Datacube, role: str, read_only: bool = False) -> None:
        if read_only:
            self._run_sql(dc, "ows_schema/grants/read_only", role=role)
        else:
            self._run_sql(dc, "ows_schema/grants/read_write", role=role)

    # Spatiotemporal index update method (e.g. refresh materialised views)
    @override
    def update_geotemporal_index(self, dc: Datacube):
        # Native ODC geotemporal index used in postgis driver.
        pass

    @override
    def create_range_entry(self, layer: OWSNamedLayer, cache: dict[LayerSignature, list[str]]) -> None:
        create_range_entry_impl(layer, cache)

    @override
    def get_ranges(self, layer: OWSNamedLayer) -> LayerExtent | None:
        return get_ranges_impl(layer)

    def _query(self,
               layer: OWSNamedLayer,
               times: Iterable[TimeSearchTerm] | None = None,
               geom: Geometry | None = None,
               products: Iterable[Product] | None = None
               ) -> dict[str, Any]:
        query: dict[str, Any] = {}
        if geom:
            if geom.crs and geom.crs in layer.dc.index.spatial_indexes():
                query["geopolygon"] = geom
            else:
                # Default to 4326 and take a long hard look at yourself.
                prepared_geom = self._prep_geom(layer, geom)
                assert prepared_geom is not None
                geopoly = prepared_geom.to_crs("epsg:4326")
                geopoly = Geometry(fix_shape(geopoly.geom), crs="epsg:4326")
                query["geopolygon"] = geopoly
        if products is not None:
            query["product"] = [p.name for p in products]
        if times is not None:

            def normalise_to_dtr(unnorm: datetime.datetime | datetime.date) -> tuple[datetime.datetime, datetime.datetime]:
                if isinstance(unnorm, datetime.datetime):
                    st: datetime.datetime = default_to_utc(unnorm)
                    tmax = st + datetime.timedelta(seconds=1)
                elif isinstance(t, datetime.date):
                    st = datetime.datetime(unnorm.year, unnorm.month, unnorm.day, tzinfo=datetime.timezone.utc)
                    tmax = st + datetime.timedelta(days=1)
                else:
                    raise ValueError("Not a datetime object")
                return st, tmax

            time_args = []
            for t in times:
                if isinstance(t, (datetime.date, datetime.datetime)):
                    start, tmax = normalise_to_dtr(t)
                    time_args.append(Range(start, tmax))
                else:
                    st, et = t
                    st, _ = normalise_to_dtr(st)
                    et, _ = normalise_to_dtr(et)
                    time_args.append(Range(st, et))
            if len(time_args) > 1:
                raise ValueError("Huh?")
            query["time"] = time_args[0]
        return query

    @override
    def ds_search(self,
                  layer: OWSNamedLayer,
                  times: Iterable[TimeSearchTerm] | None = None,
                  geom: Geometry | None = None,
                  products: Iterable[Product] | None = None
                  ) -> Iterable[Dataset]:
        return layer.dc.index.datasets.search(**self._query(layer, times, geom, products))

    @override
    def dsid_search(self,
                    layer: OWSNamedLayer,
                    times: Iterable[TimeSearchTerm] | None = None,
                    geom: Geometry | None = None,
                    products: Iterable[Product] | None = None
                    ) -> Iterable[UUID]:
        for ds in layer.dc.index.datasets.search_returning(field_names=["id"],
                                                           **self._query(layer, times, geom, products)):
            yield ds.id  # type: ignore[attr-defined]

    @override
    def count(self,
              layer: OWSNamedLayer,
              times: Iterable[TimeSearchTerm] | None = None,
              geom: Geometry | None = None,
              products: Iterable[Product] | None = None
              ) -> int:
        return layer.dc.index.datasets.count(**self._query(layer, times, geom, products))

    @override
    def extent(self,
               layer: OWSNamedLayer,
               times: Iterable[TimeSearchTerm] | None = None,
               geom: Geometry | None = None,
               products: Iterable[Product] | None = None,
               crs: CRS | None = None
               ) -> Geometry | None:
        if crs is None:
            crs = CRS("epsg:4326")
        return layer.dc.index.datasets.spatial_extent(
            self.dsid_search(layer, times=times, geom=geom, products=products),
            crs=crs
        )

    def _run_sql(self, dc: Datacube, path: str, **params: str) -> bool:
        return run_sql(dc, self.name, path, **params)


pgisdriverlock = Lock()


class OWSPostgisIndexDriver(OWSAbstractIndexDriver):
    _driver = None
    @classmethod
    @override
    def ows_index_class(cls) -> type[OWSAbstractIndex]:
        return OWSPostgisIndex

    @classmethod
    @override
    def ows_index(cls) -> OWSAbstractIndex:
        with pgisdriverlock:
            if cls._driver is None:
                cls._driver = OWSPostgisIndex()
        return cls._driver


def ows_index_driver_init() -> OWSPostgisIndexDriver:
    return OWSPostgisIndexDriver()



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0


import copy
import os


from .pure_styles import (
    style_s2_pure_aerosol, style_s2_pure_blue, style_s2_pure_green,
    style_s2_pure_narrow_nir, style_s2_pure_nir, style_s2_pure_red,
    style_s2_pure_redge_1, style_s2_pure_redge_2, style_s2_pure_redge_3,
    style_s2_pure_swir1, style_s2_pure_swir2)

if os.environ.get("DATACUBE_OWS_CFG", "").startswith("local"):
    cfgbase = "local_cfg"
    trans_dir = "."
else:
    cfgbase = "config"
    trans_dir = "/code"

legend_idx_0_1_5ticks = {"begin": "0.0", "end": "1.0", "ticks_every": "0.2"}

s2_nrt_fmask = [
    {
        "band": "fmask",
        "values": [0, 2, 3],
        "invert": True,
    },
    {
        "band": "land",
        "invert": True,
        "values": [1],
    },
]

s2_provisional_oa_fmask = [
    {
        "band": "oa_fmask",
        "values": [0, 2, 3],
        "invert": True,
    },
    {
        "band": "land",
        "invert": True,
        "values": [1],
    }
]

s2_c3_cloudless_mask = [
    {
        "band": "s2cloudless_mask",
        "values": [0, 2, 3],
        "invert": True,
    },
    {
        "band": "land",
        "invert": True,
        "values": [1],
    }
]


style_s2_simple_rgb = {
    "name": "simple_rgb",
    "title": "Simple RGB",
    "abstract": "Simple true-colour image, using the red, green and blue bands",
    "components": {"red": {"red": 1.0}, "green": {"green": 1.0}, "blue": {"blue": 1.0}},
    "scale_range": [0.0, 3000.0],
}
style_s2_irg = {
    "name": "infrared_green",
    "title": "False colour - Green, SWIR, NIR",
    "abstract": "False Colour image with SWIR1->Red, NIR->Green, and Green->Blue",
    "components": {
        "red": {"nbart_swir_2": 1.0},
        "green": {"nbart_nir_1": 1.0},
        "blue": {"nbart_green": 1.0},
    },
    "scale_range": [0.0, 3000.0],
}

style_s2_ndvi = {
    "name": "ndvi",
    "title": "Normalised Difference Vegetation Index - Red, NIR",
    "abstract": "Normalised Difference Vegetation Index - a derived index that correlates well with the existence of vegetation",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {"band1": "nir", "band2": "red"},
    },
    "needed_bands": ["red", "nir"],
    "color_ramp": [
        {"value": -0.0, "color": "#8F3F20", "alpha": 0.0},
        {"value": 0.0, "color": "#8F3F20", "alpha": 1.0},
        {"value": 0.1, "color": "#A35F18"},
        {"value": 0.2, "color": "#B88512"},
        {"value": 0.3, "color": "#CEAC0E"},
        {"value": 0.4, "color": "#E5D609"},
        {"value": 0.5, "color": "#FFFF0C"},
        {"value": 0.6, "color": "#C3DE09"},
        {"value": 0.7, "color": "#88B808"},
        {"value": 0.8, "color": "#529400"},
        {"value": 0.9, "color": "#237100"},
        {"value": 1.0, "color": "#114D04"},
    ],
    "pq_masks": s2_nrt_fmask,
    "legend": legend_idx_0_1_5ticks,
    "multi_date": [
        {
            "allowed_count_range": [2, 2],
            "preserve_user_date_order": True,
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_delta"
            },
            "mpl_ramp": "RdYlBu",
            "range": [-1.0, 1.0],
            "pq_masks": s2_nrt_fmask,
            "legend": {
                "begin": "-1.0",
                "end": "1.0",
                "ticks": [
                    "-1.0",
                    "0.0",
                    "1.0",
                ],
            },
            "feature_info_label": "ndvi_delta",
        }
    ],
}

style_s2_provisional_ndvi = copy.deepcopy(style_s2_ndvi)
style_s2_provisional_ndvi["pq_masks"] = s2_provisional_oa_fmask
style_s2_provisional_ndvi["multi_date"][0]["pq_masks"] = s2_provisional_oa_fmask

style_s2_c3_ndvi = copy.deepcopy(style_s2_ndvi)
style_s2_c3_ndvi["pq_masks"] = s2_c3_cloudless_mask
style_s2_c3_ndvi["multi_date"][0]["pq_masks"] = s2_c3_cloudless_mask

style_s2_ndwi = {
    "name": "ndwi",
    "title": "Normalised Difference Water Index - Green, NIR",
    "abstract": "Normalised Difference Water Index - a derived index that correlates well with the existence of water (McFeeters 1996)",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {"band1": "green", "band2": "nir"},
    },
    "needed_bands": ["green", "nir"],
    "color_ramp": [
        {"value": -0.1, "color": "#f7fbff", "alpha": 0.0},
        {
            "value": 0.0,
            "color": "#d8e7f5",
        },
        {"value": 0.1, "color": "#b0d2e8"},
        {
            "value": 0.2,
            "color": "#73b3d8",
        },
        {"value": 0.3, "color": "#3e8ec4"},
        {
            "value": 0.4,
            "color": "#1563aa",
        },
        {
            "value": 0.5,
            "color": "#08306b",
        },
    ],
    "pq_masks": s2_nrt_fmask,
    "legend": {
        "begin": "0.0",
        "end": "0.5",
        "decimal_places": 1,
        "ticks": ["0.0", "0.2", "0.4", "0.5"],
        "tick_labels": {
            "0.0": {"prefix": "<"},
            "0.2": {"label": "0.2"},
            "0.4": {"label": "0.4"},
            "0.5": {"prefix": ">"},
        }
    },
    "multi_date": [
        {
            "allowed_count_range": [2, 2],
            "preserve_user_date_order": True,
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_delta"
            },
            "mpl_ramp": "RdYlBu",
            "range": [-1.0, 1.0],
            "pq_masks": s2_nrt_fmask,
            "legend": {
                "begin": "-1.0",
                "end": "1.0",
                "ticks": [
                    "-1.0",
                    "-0.0",
                    "1.0",
                ],
            },
            "feature_info_label": "ndwi_delta",
        }
    ],
}

style_s2_provisional_ndwi = copy.deepcopy(style_s2_ndwi)
style_s2_provisional_ndwi["pq_masks"] = s2_provisional_oa_fmask
style_s2_provisional_ndwi["multi_date"][0]["pq_masks"] = s2_provisional_oa_fmask

style_s2_c3_ndwi = copy.deepcopy(style_s2_ndwi)
style_s2_c3_ndwi["pq_masks"] = s2_c3_cloudless_mask
style_s2_c3_ndwi["multi_date"][0]["pq_masks"] = s2_c3_cloudless_mask

style_s2_mndwi = {
    # Cannot reuse landsat as we need swir_2 to landsat's swir_1
    "name": "mndwi",
    "title": "Modified Normalised Difference Water Index - Green, SWIR",
    "abstract": "Modified Normalised Difference Water Index - a derived index that correlates well with the existence of water (Xu 2006)",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {"band1": "nbart_green", "band2": "nbart_swir_2"},
    },
    "needed_bands": ["nbart_green", "nbart_swir_2"],
    "color_ramp": [
        {"value": -0.1, "color": "#f7fbff", "alpha": 0.0},
        {"value": 0.0, "color": "#d8e7f5"},
        {"value": 0.2, "color": "#b0d2e8"},
        {"value": 0.4, "color": "#73b3d8"},
        {"value": 0.6, "color": "#3e8ec4"},
        {"value": 0.8, "color": "#1563aa"},
        {"value": 1.0, "color": "#08306b"},
    ],
    "pq_masks": s2_nrt_fmask,
    "legend": legend_idx_0_1_5ticks,
    "multi_date": [
        {
            "allowed_count_range": [2, 2],
            "preserve_user_date_order": True,
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_delta"
            },
            "mpl_ramp": "RdYlBu",
            "range": [-1.0, 1.0],
            "pq_masks": s2_nrt_fmask,
            "legend": {
                "begin": "-1.0",
                "end": "1.0",
                "ticks": [
                    "-1.0",
                    "0.0",
                    "1.0",
                ],
            },
            "feature_info_label": "mndwi_delta",
        }
    ],
}

style_s2_provisional_mndwi = copy.deepcopy(style_s2_mndwi)
style_s2_provisional_mndwi["pq_masks"] = s2_provisional_oa_fmask
style_s2_provisional_mndwi["multi_date"][0]["pq_masks"] = s2_provisional_oa_fmask

style_s2_c3_mndwi = copy.deepcopy(style_s2_mndwi)
style_s2_c3_mndwi["pq_masks"] = s2_c3_cloudless_mask
style_s2_c3_mndwi["multi_date"][0]["pq_masks"] = s2_c3_cloudless_mask

style_s2_ndci = {
    "name": "ndci",
    "title": "Normalised Difference Chlorophyll Index - Red Edge, Red",
    "abstract": "Normalised Difference Chlorophyll Index - a derived index that correlates well with the existence of chlorophyll",
    "index_function": {
        "function": "datacube_ows.band_utils.sentinel2_ndci",
        "mapped_bands": True,
        "kwargs": {
            "b_red_edge": "nbart_red_edge_1",
            "b_red": "nbart_red",
            "b_green": "nbart_green",
            "b_swir": "nbart_swir_2",
        },
    },
    "needed_bands": ["nbart_red_edge_1", "nbart_red", "nbart_green", "nbart_swir_2"],
    "color_ramp": [
        {
            "value": -0.1,
            "color": "#1696FF",
        },
        {"value": -0.1, "color": "#1696FF"},
        {
            "value": 0.0,
            "color": "#00FFDF",
        },
        {
            "value": 0.1,
            "color": "#FFF50E",
        },
        {
            "value": 0.2,
            "color": "#FFB50A",
        },
        {
            "value": 0.4,
            "color": "#FF530D",
        },
        {
            "value": 0.5,
            "color": "#FF0000",
        },
    ],
    "legend": {
        "begin": "-0.1",
        "end": "0.5",
        "ticks_every": "0.1",
        "units": "unitless",
        "tick_labels": {"-0.1": {"prefix": "<"}, "0.5": {"prefix": ">"}},
    },
}

style_s2_nbr = {
    "name": "nbr",
    "title": "Normalised Burn Ratio - NIR, SWIR",
    "abstract": "Normalised Burn Ratio - a derived index that that uses the differences in the way health green vegetation and burned vegetation reflect light to find burned area",
    "index_function": {
        "function": "datacube_ows.band_utils.norm_diff",
        "mapped_bands": True,
        "kwargs": {"band1": "nbart_nir_1", "band2": "nbart_swir_3"},
    },
    "needed_bands": ["nbart_nir_1", "nbart_swir_3"],
    "color_ramp": [
        {
            "value": -1.0,
            "color": "#67001F",
            "alpha": 0.0,
        },
        {
            "value": -1.0,
            "color": "#67001F",
        },
        {
            "value": -0.8,
            "color": "#B2182B",
        },
        {"value": -0.4, "color": "#D6604D"},
        {"value": -0.2, "color": "#F4A582"},
        {"value": -0.1, "color": "#FDDBC7"},
        {
            "value": 0,
            "color": "#F7F7F7",
        },
        {"value": 0.2, "color": "#D1E5F0"},
        {"value": 0.4, "color": "#92C5DE"},
        {"value": 0.6, "color": "#4393C3"},
        {"value": 0.9, "color": "#2166AC"},
        {
            "value": 1.0,
            "color": "#053061",
        },
    ],
    "pq_masks": s2_nrt_fmask,
    "legend": {
        "show_legend": True,
        "begin": "-1.0",
        "end": "1.0",
        "ticks_every": "1.0",
        "decimal_places": 0,
        "tick_labels": {"-1.0": {"prefix": "<"}, "1.0": {"suffix": ">"}},
    },
    "multi_date": [
        {
            "allowed_count_range": [2, 2],
            "preserve_user_date_order": True,
            "aggregator_function": {
                "function": "datacube_ows.band_utils.multi_date_delta"
            },
            "color_ramp": [
                {"value": -0.5, "color": "#768642", "alpha": 0.0},
                {"value": -0.5, "color": "#768642"},
                {
                    "value": -0.25,
                    "color": "#768642",
                    "alpha": 1.0,
                },
                {"value": -0.25, "color": "#a4bd5f"},
                {"value": -0.1, "color": "#a4bd5f"},
                {"value": -0.1, "color": "#00e05d"},
                {"value": 0.1, "color": "#00e05d"},
                {"value": 0.1, "color": "#fdf950"},
                {"value": 0.27, "color": "#fdf950"},
                {"value": 0.27, "color": "#ffae52"},
                {"value": 0.44, "color": "#ffae52"},
                {"value": 0.44, "color": "#ff662e"},
                {"value": 0.66, "color": "#ff662e"},
                {"value": 0.66, "color": "#ad28cc"},
                {"value": 0.88, "color": "#ad28cc"},
            ],
            "legend": {
                "begin": "-0.5",
                "end": "0.88",
                "ticks": [
                    "-0.5",
                    "-0.25",
                    "-0.1",
                    "0.1",
                    "0.27",
                    "0.44",
                    "0.66",
                    "0.88",
                ],
                "tick_labels": {
                    "-0.5": {"label": "<-0.5"},
                    "-0.25": {"label": "-0.25"},
                    "-0.1": {"label": "-0.1"},
                    "0.1": {"label": "0.1"},
                    "0.27": {"label": "0.27"},
                    "0.44": {"label": "0.44"},
                    "0.66": {"label": "0.66"},
                    "0.88": {"label": ">1.30"},
                },
            },
            # The multi-date color ramp.  May be defined as an explicit colour ramp, as shown above for the single
            # date case; or may be defined with a range and unscaled color ramp as shown here.
            #
            # The range specifies the min and max values for the color ramp.  Required if an explicit color
            # ramp is not defined.
            # "range": [-1.0, 1.0],
            # The name of a named matplotlib color ramp.
            # Reference here: https://matplotlib.org/examples/color/colormaps_reference.html
            # Only used if an explicit colour ramp is not defined.  Optional - defaults to a simple (but
            # kind of ugly) blue-to-red rainbow ramp.
            # "mpl_ramp": "RdBu",
            # The feature info label for the multi-date index value.
            "feature_info_label": "nbr_delta",
        }
    ],
}

style_s2_provisional_nbr = copy.deepcopy(style_s2_nbr)
style_s2_provisional_nbr["pq_masks"] = s2_provisional_oa_fmask

style_s2_c3_nbr = copy.deepcopy(style_s2_nbr)
style_s2_c3_nbr["pq_masks"] = s2_c3_cloudless_mask

fmask_bits = [
    {
        "title": "No Data",
        "abstract": "",
        "values": [
            0,    # nodata
        ],
        "alpha": 0.0,
        "color": "#FFFFFF",
    },
    {
        "title": "Clear",
        "abstract": "",
        "values": [
            1,    # clear/valid
        ],
        "color": "#84A278",
    },
    {
        "title": "Cloud",
        "abstract": "",
        "values": [
            2,    # cloud
        ],
        "color": "#D0CFCE",
    },
    {
        "title": "Shadow",
        "abstract": "",
        "values": [
            3    # shadow
        ],
        "color": "#464633",
    },
    {
        "title": "Snow",
        "abstract": "",
        "values": [
            4    # snow
        ],
        "color": "#E0EDFF",
    },
    {
        "title": "Water",
        "abstract": "",
        "values": [
            5    # water
        ],
        "color": "#475B74",
    },
]

s2cloudless_mask_bits = [
    {
        "title": "No Data",
        "abstract": "",
        "values": [
            0,    # nodata
        ],
        "alpha": 0.0,
        "color": "#FFFFFF",
    },
    {
        "title": "Clear",
        "abstract": "",
        "values": [
            1,    # clear/valid
        ],
        "color": "#84A278",
    },
    {
        "title": "Cloud",
        "abstract": "",
        "values": [
            2,    # cloud
        ],
        "color": "#D0CFCE",
    },
]

style_s2_fmask = {
    "name": "fmask",
    "title": "Fmask Classification",
    "abstract": "Fmask (Function of mask) is used for automated clouds, cloud shadows, snow, and water masking for Landsats 4-8 and Sentinel 2 images.",
    "include_in_feature_info": False,
    "needed_bands": ["fmask"],
    "value_map": {
        "fmask": fmask_bits
    }
}

style_s2_oa_fmask = {
    "name": "fmask",
    "title": "Fmask Classification",
    "abstract": "Fmask (Function of mask) is used for automated clouds, cloud shadows, snow, and water masking for Landsats 4-8 and Sentinel 2 images.",
    "include_in_feature_info": False,
    "needed_bands": ["oa_fmask"],
    "value_map": {
        "oa_fmask": fmask_bits
    }
}

style_s2_cloudless_mask = {
    "name": "s2cloudless_mask",
    "title": "S2 Cloudless Mask Classification",
    "abstract": "S2 Cloudless Mask is used for automated cloud masking Sentinel 2 images.",
    "include_in_feature_info": False,
    "needed_bands": ["s2cloudless_mask"],
    "value_map": {
        "s2cloudless_mask": s2cloudless_mask_bits
    }
}

style_s2_blackcloud = {
    "name": "s2dark_cloud",
    "title": "S2 Darken clouds",
    "abstract": "S2 Cloudless Probabilities given for s2cloudless_mask classification",
    "include_in_feature_info": False,
    "additional_bands": ["s2cloudless_prob", "red", "green", "blue"],
    "components": {
        "red": {
            "function": f"{cfgbase}.s2_c3_local_cfg.utils.black_cloud",
            "mapped_bands": True,
            "kwargs": {
                "band": "red",
                "scale_from": [0, 3000]
            }
        },
        "green": {
            "function": f"{cfgbase}.s2_c3_local_cfg.utils.black_cloud",
            "mapped_bands": True,
            "kwargs": {
                "band": "green",
                "scale_from": [0, 3000]
            }
        },
        "blue": {
            "function": f"{cfgbase}.s2_c3_local_cfg.utils.black_cloud",
            "mapped_bands": True,
            "kwargs": {
                "band": "blue",
                "scale_from": [0, 3000]
            }
        },
    }
}
style_s2_cloudless_prob = {
    "name": "s2cloudless_prob",
    "title": "S2 Cloudless Mask Probability",
    "abstract": "S2 Cloudless Probabilities given for s2cloudless_mask classification",
    "include_in_feature_info": False,
    "additional_bands": ["s2cloudless_prob"],
    "components": {
        "red": {
            "red": 1.0,
            "scale_range": (0, 3000),
        },
        "green": {
            "green": 1.0,
            "scale_range": (0, 3000),
        },
        "blue": {
            "blue": 1.0,
            "scale_range": (0, 3000),
        },
        "alpha": {
            "function": f"{cfgbase}.s2_c3_local_cfg.utils.invert_alpha"
        }
    }
}

styles_s2_list = [
    style_s2_simple_rgb,
    style_s2_irg,
    style_s2_ndvi,
    style_s2_ndwi,
    style_s2_mndwi,
    style_s2_ndci,
    style_s2_nbr,
    style_s2_pure_aerosol,
    style_s2_pure_blue,
    style_s2_pure_green,
    style_s2_pure_red,
    style_s2_pure_redge_1,
    style_s2_pure_redge_2,
    style_s2_pure_redge_3,
    style_s2_pure_nir,
    style_s2_pure_narrow_nir,
    style_s2_pure_swir1,
    style_s2_pure_swir2,
    style_s2_fmask,
]

styles_s2_c3_list = [
    style_s2_simple_rgb,
    style_s2_irg,
    style_s2_c3_ndvi,
    style_s2_c3_ndwi,
    style_s2_c3_mndwi,
    style_s2_ndci,
    style_s2_c3_nbr,
    style_s2_pure_aerosol,
    style_s2_pure_blue,
    style_s2_pure_green,
    style_s2_pure_red,
    style_s2_pure_redge_1,
    style_s2_pure_redge_2,
    style_s2_pure_redge_3,
    style_s2_pure_nir,
    style_s2_pure_narrow_nir,
    style_s2_pure_swir1,
    style_s2_pure_swir2,
    style_s2_fmask,
    style_s2_cloudless_mask,
    style_s2_cloudless_prob,
    style_s2_blackcloud,
]


styles_s2_provisional_list = [
    style_s2_simple_rgb,
    style_s2_irg,
    style_s2_provisional_ndvi,
    style_s2_provisional_ndwi,
    style_s2_provisional_mndwi,
    style_s2_ndci,
    style_s2_provisional_nbr,
    style_s2_pure_aerosol,
    style_s2_pure_blue,
    style_s2_pure_green,
    style_s2_pure_red,
    style_s2_pure_redge_1,
    style_s2_pure_redge_2,
    style_s2_pure_redge_3,
    style_s2_pure_nir,
    style_s2_pure_narrow_nir,
    style_s2_pure_swir1,
    style_s2_pure_swir2,
    style_s2_oa_fmask,
]



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0


from datacube_ows.band_utils import scalable


def invert_alpha(data):
    return -255 * data["s2cloudless_prob"] + 255

@scalable
def black_cloud(data, band, band_mapper=None):
    cloudless_prob = 1.0 - data["s2cloudless_prob"]
    if band_mapper:
        band = band_mapper(band)
    return data[band] * cloudless_prob



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0




import os

if os.environ.get("DATACUBE_OWS_CFG", "").startswith("local"):
    cfgbase = "local_cfg"
else:
    cfgbase = "config"


reslim_wms_min_zoom_15_cache_rules = {}

bands_landcover = {
    "level3": [],
    "level4": [],
}

bands_eds = {
    "lifeform_veg_cat_l4a": ["lifeform"],
    "canopyco_veg_cat_l4d": ["canopy_cover"],
    "watersea_veg_cat_l4a_au": ["water_seasonality"],
    "waterper_wat_cat_l4d_au": ["water_persistence"],
    "baregrad_phy_cat_l4d_au": ["bare_gradation"],
}

style_c2_level3_test = {
    "name": "level3_test",
    "title": "Basic - 6 classes",
    "abstract": "Standardised colouring of Level 3 land cover classes",
    "value_map": {
        "level3": [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': "Cultivated Terrestrial Vegetation", 'abstract': "", 'values': [111], 'color': '#ACBC2D', 'alpha': 1},
            {'title': "Natural Terrestrial Vegetation", 'abstract': "", 'values': [112], 'color': '#0E7912', 'alpha': 1},
            {'title': "Natural Aquatic Vegetation", 'abstract': "", 'values': [124], 'color': '#1EBF79', 'alpha': 1},
            {'title': "Artificial Surface", 'abstract': "", 'values': [215], 'color': '#DA5C69', 'alpha': 1},
            {'title': "Natural Bare Surface", 'abstract': "", 'values': [216], 'color': '#F3AB69', 'alpha': 1},
            {'title': "Water", 'abstract': "", 'values': [220], 'color': '#4D9FDC', 'alpha': 1}
        ]
    },
    "legend": {"width": 3.0, "height": 1.5},
    "multi_date": [
        {
            "animate": False,
            "preserve_user_date_order": True,
            "allowed_count_range": [2, 2],
            "value_map": {
                "level3": [
                    {'title': "Artifical Surface (unchanged)", 'abstract': "", 'values': [[216], [216]],
                     'color': '#801010', 'alpha': 1},
                    {'title': "Other -> Artifical Surface", 'abstract': "", 'values': [[], [216]],
                     'color': '#C02030', 'alpha': 1},
                    {'title': "Natural Bare Surface (unchanged)", 'abstract': "", 'values': [[215], [215]],
                     'color': '#404050', 'alpha': 1},
                    {'title': "Other -> Natural Bare Surface", 'abstract': "", 'values': [[], [215]],
                     'color': '#70707A', 'alpha': 1},

                    {'title': "Water/Aquatic veg", 'abstract': "", 'values': [[], [124,215,216,220]],
                     'color': '#4040A0', 'alpha': 1},

                    {'title': "Cultivated Terrestrial Veg -> Natural Veg", 'abstract': "", 'values': [[111], [112]],
                     'color': '#FFFFFF', 'alpha': 1},
                    {'title': "Natural Terrestrial Veg (unchanged)", 'abstract': "", 'values': [[112], [112]],
                     'color': '#109030', 'alpha': 1},
                    {'title': "Other -> Terrestrial Natural Veg", 'abstract': "", 'values': [[], [112]],
                     'color': '#80E080', 'alpha': 1},
                    {'title': "Natural Terrestrial Veg -> Cultivated Veg", 'abstract': "", 'values': [[112], [111]], 'color': '#80E0B0', 'alpha': 1},
                    {'title': "Other -> Cultivated Veg", 'abstract': "", 'values': [[], [111]], 'color': '#B0E0D0', 'alpha': 1},

                    {'title': "Everything else", 'abstract': "", 'values': [[], []], 'color': '#00FFFF', 'alpha': 1},
                ]
            },
            "legend": {
                "width": 3.5,
                "height": 2.8
            }
        }
    ]
}

style_c2_level3 = {
    "name": "level3",
    "title": "Basic - 6 classes",
    "abstract": "Standardised colouring of Level 3 land cover classes",
    "value_map": {
        "level3": [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': "Cultivated Terrestrial Vegetation", 'abstract': "", 'values': [111], 'color': '#ACBC2D', 'alpha': 1},
            {'title': "Natural Terrestrial Vegetation", 'abstract': "", 'values': [112], 'color': '#0E7912', 'alpha': 1},
            {'title': "Natural Aquatic Vegetation", 'abstract': "", 'values': [124], 'color': '#1EBF79', 'alpha': 1},
            {'title': "Artificial Surface", 'abstract': "", 'values': [215], 'color': '#DA5C69', 'alpha': 1},
            {'title': "Natural Bare Surface", 'abstract': "", 'values': [216], 'color': '#F3AB69', 'alpha': 1},
            {'title': "Water", 'abstract': "", 'values': [220], 'color': '#4D9FDC', 'alpha': 1}
        ]
    },
    "legend": {"width": 3.0, "height": 1.5},
    "multi_date": [
        {
            "animate": False,
            "preserve_user_date_order": True,
            "allowed_count_range": [2, 2],
            "value_map": {
                "level3": [
                    {'title': "Artifical Surface (unchanged)", 'abstract': "", 'values': [[216], [216]],
                     'color': '#801010', 'alpha': 1},
                    {'title': "Other -> Artifical Surface", 'abstract': "", 'values': [[], [216]],
                     'color': '#C02030', 'alpha': 1},
                    {'title': "Natural Bare Surface (unchanged)", 'abstract': "", 'values': [[215], [215]],
                     'color': '#404050', 'alpha': 1},
                    {'title': "Other -> Natural Bare Surface", 'abstract': "", 'values': [[], [215]],
                     'color': '#70707A', 'alpha': 1},

                    {'title': "Water/Aquatic veg", 'abstract': "", 'values': [[], [124,215,216,220]],
                     'color': '#4040A0', 'alpha': 1},
                    {'title': "Cultivated Terrestrial Veg -> Natural Veg", 'abstract': "", 'values': [[111], [112]],
                     'color': '#B0E080', 'alpha': 1},
                    {'title': "Natural Terrestrial Veg (unchanged)", 'abstract': "", 'values': [[112], [112]],
                     'color': '#109030', 'alpha': 1},
                    {'title': "Natural Terrestrial Veg -> Cultivated Veg", 'abstract': "", 'values': [[112], [111]], 'color': '#80E0B0', 'alpha': 1},
                    {'title': "Other -> Terrestrial Veg", 'abstract': "", 'values': [[], [112]],
                     'color': '#80E080', 'alpha': 1},
                    {'title': "Other -> Cultivated Veg", 'abstract': "", 'values': [[], [111]], 'color': '#B0E0D0', 'alpha': 1},
                    {'title': "Everything else", 'abstract': "", 'values': [[], []], 'color': '#00FFFF', 'alpha': 1},
                ]
            },
            "legend": {
                "width": 3.5,
                "height": 2.8
            }
        }
    ]
}

style_c2_level4_lifeform = {
    "name": "lifeform",
    "title": "Lifeform",
    "abstract": "Lifeform (L4): woody / herbaceous",
    "value_map": {
        "lifeform_veg_cat_l4a": [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': "Woody", 'abstract': "", 'values': [1], 'color': '#0E7912', 'alpha': 1},
            {'title': "Herbaceous", 'abstract': "", 'values': [2], 'color': '#77A71E', 'alpha': 1}
        ]
    },
    # "pq_masks": [
    #     {
    #         "band": "land",
    #         "invert": True,
    #         "enum": 0,
    #     }
    # ],
}


style_c2_level4_canopyco = {
    "name": "canopyco",
    "title": "Canopy Cover",
    "abstract": "Canopy Cover (L4)",
    "value_map": {
        'canopyco_veg_cat_l4d': [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': "Closed Vegetation", 'abstract': "(> 65 %)", 'values': [10], 'color': '#0E7912', 'alpha': 1},
            {'title': "Open Vegetation", 'abstract': "(40 to 65 %)", 'values': [12], 'color': '#2D8D2F', 'alpha': 1},
            {'title': "Open Vegetation", 'abstract': "(15 to 40 %)", 'values': [13], 'color': '#50A052', 'alpha': 1},
            {'title': "Sparse Vegetation", 'abstract': "(4 to 15 %)", 'values': [15], 'color': '#75B476', 'alpha': 1},
            {'title': "Scattered Vegetation", 'abstract': "(1 to 4 %)", 'values': [16], 'color': '#9AC79C', 'alpha': 1}
        ]
    },
    # "pq_masks": [
    #     {
    #         "band": "land",
    #         "invert": True,
    #         "enum": 0,
    #     }
    # ],
}


style_c2_level4_watersea = {
    "name": "watersea",
    "title": "Water Seasonality",
    "abstract": "Water Seasonality (L4) in vegetated areas",
    "value_map": {
        "watersea_veg_cat_l4a_au": [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': "> 3 months", 'abstract': "(semi) permanent", 'values': [1], 'color': '#19AD6D', 'alpha': 1},
            {'title': "< 3 months", 'abstract': "temporary or seasonal", 'values': [2], 'color': '#B0DAC9', 'alpha': 1}
        ]
    },
}


style_c2_level4_waterper = {
    "name": "waterper",
    "title": "Water Persistence",
    "abstract": "Water persistence in non-vegetated areas",
    "value_map": {
        "waterper_wat_cat_l4d_au": [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': "Perennial", 'abstract': "(> 9 months)", 'values': [1], 'color': '#1b55ba', 'alpha': 1},
            {'title': "Non-perennial", 'abstract': "(7 to 9 months)", 'values': [7], 'color': '#3479C9', 'alpha': 1},
            {'title': "Non-perennial", 'abstract': "(4 to 6 months)", 'values': [8], 'color': '#4F9DD9', 'alpha': 1},
            {'title': "Non-perennial", 'abstract': "(1 to 3 months)", 'values': [9], 'color': '#71CAFD', 'alpha': 1}
        ]
    }
}


style_c2_level4_baregrad = {
    "name": "baregrad",
    "title": "Bare Gradation",
    "abstract": "Level of vegetation in predominantly bare areas",
    "value_map": {
        "baregrad_phy_cat_l4d_au": [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': "Sparsely Vegetated", 'abstract': "", 'values': [10], 'color': '#FFE68C', 'alpha': 1},
            {'title': "Very Sparsely Vegetated", 'abstract': "", 'values': [12], 'color': '#FAD26E', 'alpha': 1},
            {'title': "Bare Areas, Unvegetated", 'abstract': "", 'values': [15], 'color': '#F3AB69', 'alpha': 1}
        ]
    },
    # "pq_masks": [
    #     {
    #         "band": "land",
    #         "invert": True,
    #         "enum": 0,
    #     }
    # ],
}


style_c2_level4 = {
    "name": "level4",
    "title": "Detailed - 80 classes",
    "abstract": "Standardised colouring of Level 4 land cover classes",
    "value_map": {
        "level4": [
            {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
            {'title': 'Cultivated Terrestrial Vegetated', 'abstract': '', 'values': [1], 'color': '#97bb1a', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Woody', 'abstract': '', 'values': [2], 'color': '#97bb1a', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Herbaceous', 'abstract': '', 'values': [3], 'color': '#d1e033', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [4], 'color': '#c5a847', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [5], 'color': '#cdb54b', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [6], 'color': '#d5c14f', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [7], 'color': '#e4d26c', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [8], 'color': '#f2e38a', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [9], 'color': '#c5a847', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Woody Open (40 to 65 %)', 'abstract': '', 'values': [10], 'color': '#cdb54b', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Woody Open (15 to 40 %)', 'abstract': '', 'values': [11], 'color': '#d5c14f', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '', 'values': [12], 'color': '#e4d26c', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '', 'values': [13], 'color': '#f2e38a', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '', 'values': [14], 'color': '#e4e034', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '', 'values': [15], 'color': '#ebe854', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '', 'values': [16], 'color': '#f2f07f', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '', 'values': [17], 'color': '#f9f7ae', 'alpha': 1},
            {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '', 'values': [18], 'color': '#fffede', 'alpha': 1},
            {'title': '', 'abstract': '', 'values': [19], 'color': '#0e7912', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Woody', 'abstract': '', 'values': [20], 'color': '#1ab157', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Herbaceous', 'abstract': '', 'values': [21], 'color': '#5eb31f', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [22], 'color': '#0e7912', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [23], 'color': '#2d8d2f', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [24], 'color': '#50a052', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [25], 'color': '#75b476', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [26], 'color': '#9ac79c', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [27], 'color': '#0e7912', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Woody Open (40 to 65 %)', 'abstract': '', 'values': [28], 'color': '#2d8d2f', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Woody Open (15 to 40 %)', 'abstract': '', 'values': [29], 'color': '#50a052', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '', 'values': [30], 'color': '#75b476', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '', 'values': [31], 'color': '#9ac79c', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '', 'values': [32], 'color': '#77a71e', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '', 'values': [33], 'color': '#88b633', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '', 'values': [34], 'color': '#99c450', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '', 'values': [35], 'color': '#aad471', 'alpha': 1},
            {'title': 'Natural Terrestrial Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '', 'values': [36], 'color': '#bae292', 'alpha': 1},
            {'title': '', 'abstract': '', 'values': [37], 'color': '#56ece7', 'alpha': 1},
            # {'title': 'Cultivated Aquatic Vegetated: Woody', 'abstract': '', 'values': [38], 'color': '#3daa8c', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Herbaceous', 'abstract': '', 'values': [39], 'color': '#52e7ac', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [40], 'color': '#2bd2cb', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [41], 'color': '#49ded8', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [42], 'color': '#6ee9e4', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [43], 'color': '#95f4f0', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [44], 'color': '#bbfffc', 'alpha': 1},
            # {'title': 'Cultivated Aquatic Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [45], 'color': '#2bd2cb', 'alpha': 1},
            # {'title': 'Cultivated Aquatic Vegetated: Woody Open (40 to 65 %)', 'abstract': '', 'values': [46], 'color': '#49ded8', 'alpha': 1},
            # {'title': 'Cultivated Aquatic Vegetated: Woody Open (15 to 40 %)', 'abstract': '', 'values': [47], 'color': '#6ee9e4', 'alpha': 1},
            # {'title': 'Cultivated Aquatic Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '', 'values': [48], 'color': '#95f4f0', 'alpha': 1},
            # {'title': 'Cultivated Aquatic Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '', 'values': [49], 'color': '#bbfffc', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '', 'values': [50], 'color': '#52e7c4', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '', 'values': [51], 'color': '#71edd0', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '', 'values': [52], 'color': '#90f3dc', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '', 'values': [53], 'color': '#aff9e8', 'alpha': 1},
            {'title': 'Cultivated Aquatic Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '', 'values': [54], 'color': '#cffff4', 'alpha': 1},
            {'title': '', 'abstract': '', 'values': [55], 'color': '#1ebf79', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody', 'abstract': '', 'values': [56], 'color': '#128e94', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous', 'abstract': '', 'values': [57], 'color': '#70ea86', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [58], 'color': '#19ad6d', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [59], 'color': '#35b884', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [60], 'color': '#5dc39b', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [61], 'color': '#87ceb2', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [62], 'color': '#b0dac9', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [63], 'color': '#19ad6d', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Closed (> 65 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [64], 'color': '#19ad6d', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Closed (> 65 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [65], 'color': '#19ad6d', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Open (40 to 65 %)', 'abstract': '', 'values': [66], 'color': '#35b884', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Open (40 to 65 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [67], 'color': '#35b884', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Open (40 to 65 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [68], 'color': '#35b884', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Open (15 to 40 %)', 'abstract': '', 'values': [69], 'color': '#5dc39b', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Open (15 to 40 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [70], 'color': '#5dc39b', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Open (15 to 40 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [71], 'color': '#5dc39b', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '', 'values': [72], 'color': '#87ceb2', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Sparse (4 to 15 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [73], 'color': '#87ceb2', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Sparse (4 to 15 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [74], 'color': '#87ceb2', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '', 'values': [75], 'color': '#b0dac9', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Scattered (1 to 4 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [76], 'color': '#b0dac9', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Woody Scattered (1 to 4 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [77], 'color': '#b0dac9', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '', 'values': [78], 'color': '#27cc8b', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Closed (> 65 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [79], 'color': '#27cc8b', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Closed (> 65 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [80], 'color': '#27cc8b', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '', 'values': [81], 'color': '#42d89f', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Open (40 to 65 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [82], 'color': '#42d89f', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Open (40 to 65 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [83], 'color': '#42d89f', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '', 'values': [84], 'color': '#63e3b4', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Open (15 to 40 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [85], 'color': '#63e3b4', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Open (15 to 40 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [86], 'color': '#63e3b4', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '', 'values': [87], 'color': '#87efc9', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Sparse (4 to 15 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [88], 'color': '#87efc9', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Sparse (4 to 15 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [89], 'color': '#87efc9', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '', 'values': [90], 'color': '#abfadd', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Scattered (1 to 4 %) Water > 3 months (semi-) permenant', 'abstract': '', 'values': [91], 'color': '#abfadd', 'alpha': 1},
            {'title': 'Natural Aquatic Vegetated: Herbaceous Scattered (1 to 4 %) Water < 3 months (temporary or seasonal)', 'abstract': '', 'values': [92], 'color': '#abfadd', 'alpha': 1},
            {'title': 'Artificial Surface', 'abstract': '', 'values': [93], 'color': '#da5c69', 'alpha': 1},
            {'title': '', 'abstract': '', 'values': [94], 'color': '#f3ab69', 'alpha': 1},
            {'title': 'Natural Surface: Sparsely vegetated', 'abstract': '', 'values': [95], 'color': '#ffe68c', 'alpha': 1},
            {'title': 'Natural Surface: Very sparsely vegetated', 'abstract': '', 'values': [96], 'color': '#fad26e', 'alpha': 1},
            {'title': 'Natural Surface: Bare areas, unvegetated', 'abstract': '', 'values': [97], 'color': '#f3ab69', 'alpha': 1},
            {'title': 'Water', 'abstract': '', 'values': [98], 'color': '#4d9fdc', 'alpha': 1},
            {'title': '', 'abstract': '', 'values': [99], 'color': '#4d9fdc', 'alpha': 1},
            {'title': 'Water: Tidal area', 'abstract': '', 'values': [100], 'color': '#bbdce9', 'alpha': 1},
            {'title': 'Water: Perennial (> 9 months)', 'abstract': '', 'values': [101], 'color': '#1b55ba', 'alpha': 1},
            {'title': 'Water: Non-perennial (7 to 9 months)', 'abstract': '', 'values': [102], 'color': '#3479c9', 'alpha': 1},
            {'title': 'Water: Non-perennial (4 to 6 months)', 'abstract': '', 'values': [103], 'color': '#4f9dd9', 'alpha': 1},
            {'title': 'Water: Non-perennial (1 to 3 months)', 'abstract': '', 'values': [104], 'color': '#85cafd', 'alpha': 1},
            # {'title': 'Water: (Snow)', 'abstract': '', 'values': [105], 'color': '#fafafa', 'alpha': 1},
        ]
    },
    # "pq_masks": [
    #     {
    #         "band": "land",
    #         "invert": True,
    #         "enum": 0,
    #     }
    # ],
    "legend": {
        "show_legend": True,
        "url": "https://dea-public-data-dev.s3.ap-southeast-2.amazonaws.com/lccs/level4-web-legend.png"
    },
    "multi_date": [
        {
            "animate": False,
            "preserve_user_date_order": True,
            "allowed_count_range": [2, 2],
            "aggregator_function": {
                "function": f"{cfgbase}.lccs_local_cfg.lccs_delta.lccs_map_delta",
            },
            "value_map": {
                "level4": [
                    {'title': "", 'abstract': "", 'values': [0], 'color': '#FFFFFF', 'alpha': 0},
                    {'title': "Unchanged", 'abstract': "Changed", 'values': [255], 'color': '#000000'},
                    {'title': 'Cultivated Terrestrial Vegetated', 'abstract': '', 'values': [1], 'color': '#97bb1a',
                     'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Woody', 'abstract': '', 'values': [2],
                     'color': '#97bb1a', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Herbaceous', 'abstract': '', 'values': [3],
                     'color': '#d1e033', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [4],
                     'color': '#c5a847', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [5],
                     'color': '#cdb54b', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [6],
                     'color': '#d5c14f', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [7],
                     'color': '#e4d26c', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [8],
                     'color': '#f2e38a', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [9],
                     'color': '#c5a847', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Woody Open (40 to 65 %)', 'abstract': '',
                     'values': [10], 'color': '#cdb54b', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Woody Open (15 to 40 %)', 'abstract': '',
                     'values': [11], 'color': '#d5c14f', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '',
                     'values': [12], 'color': '#e4d26c', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '',
                     'values': [13], 'color': '#f2e38a', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '',
                     'values': [14], 'color': '#e4e034', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '',
                     'values': [15], 'color': '#ebe854', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '',
                     'values': [16], 'color': '#f2f07f', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '',
                     'values': [17], 'color': '#f9f7ae', 'alpha': 1},
                    {'title': 'Cultivated Terrestrial Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '',
                     'values': [18], 'color': '#fffede', 'alpha': 1},
                    {'title': '', 'abstract': '', 'values': [19], 'color': '#0e7912', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Woody', 'abstract': '', 'values': [20],
                     'color': '#1ab157', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Herbaceous', 'abstract': '', 'values': [21],
                     'color': '#5eb31f', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [22],
                     'color': '#0e7912', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [23],
                     'color': '#2d8d2f', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [24],
                     'color': '#50a052', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [25],
                     'color': '#75b476', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [26],
                     'color': '#9ac79c', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [27],
                     'color': '#0e7912', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Woody Open (40 to 65 %)', 'abstract': '', 'values': [28],
                     'color': '#2d8d2f', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Woody Open (15 to 40 %)', 'abstract': '', 'values': [29],
                     'color': '#50a052', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '', 'values': [30],
                     'color': '#75b476', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '',
                     'values': [31], 'color': '#9ac79c', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '',
                     'values': [32], 'color': '#77a71e', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '',
                     'values': [33], 'color': '#88b633', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '',
                     'values': [34], 'color': '#99c450', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '',
                     'values': [35], 'color': '#aad471', 'alpha': 1},
                    {'title': 'Natural Terrestrial Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '',
                     'values': [36], 'color': '#bae292', 'alpha': 1},
                    {'title': '', 'abstract': '', 'values': [37], 'color': '#56ece7', 'alpha': 1},
                    # {'title': 'Cultivated Aquatic Vegetated: Woody', 'abstract': '', 'values': [38], 'color': '#3daa8c', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Herbaceous', 'abstract': '', 'values': [39],
                     'color': '#52e7ac', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [40],
                     'color': '#2bd2cb', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [41],
                     'color': '#49ded8', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [42],
                     'color': '#6ee9e4', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [43],
                     'color': '#95f4f0', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [44],
                     'color': '#bbfffc', 'alpha': 1},
                    # {'title': 'Cultivated Aquatic Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [45], 'color': '#2bd2cb', 'alpha': 1},
                    # {'title': 'Cultivated Aquatic Vegetated: Woody Open (40 to 65 %)', 'abstract': '', 'values': [46], 'color': '#49ded8', 'alpha': 1},
                    # {'title': 'Cultivated Aquatic Vegetated: Woody Open (15 to 40 %)', 'abstract': '', 'values': [47], 'color': '#6ee9e4', 'alpha': 1},
                    # {'title': 'Cultivated Aquatic Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '', 'values': [48], 'color': '#95f4f0', 'alpha': 1},
                    # {'title': 'Cultivated Aquatic Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '', 'values': [49], 'color': '#bbfffc', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '',
                     'values': [50], 'color': '#52e7c4', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '',
                     'values': [51], 'color': '#71edd0', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '',
                     'values': [52], 'color': '#90f3dc', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '',
                     'values': [53], 'color': '#aff9e8', 'alpha': 1},
                    {'title': 'Cultivated Aquatic Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '',
                     'values': [54], 'color': '#cffff4', 'alpha': 1},
                    {'title': '', 'abstract': '', 'values': [55], 'color': '#1ebf79', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody', 'abstract': '', 'values': [56], 'color': '#128e94',
                     'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Herbaceous', 'abstract': '', 'values': [57],
                     'color': '#70ea86', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Closed (> 65 %)', 'abstract': '', 'values': [58],
                     'color': '#19ad6d', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Open (40 to 65 %)', 'abstract': '', 'values': [59],
                     'color': '#35b884', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Open (15 to 40 %)', 'abstract': '', 'values': [60],
                     'color': '#5dc39b', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Sparse (4 to 15 %)', 'abstract': '', 'values': [61],
                     'color': '#87ceb2', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Scattered (1 to 4 %)', 'abstract': '', 'values': [62],
                     'color': '#b0dac9', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Closed (> 65 %)', 'abstract': '', 'values': [63],
                     'color': '#19ad6d', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Closed (> 65 %) Water > 3 months (semi-) permenant',
                     'abstract': '', 'values': [64], 'color': '#19ad6d', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Woody Closed (> 65 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [65], 'color': '#19ad6d', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Open (40 to 65 %)', 'abstract': '', 'values': [66],
                     'color': '#35b884', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Open (40 to 65 %) Water > 3 months (semi-) permenant',
                     'abstract': '', 'values': [67], 'color': '#35b884', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Woody Open (40 to 65 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [68], 'color': '#35b884', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Open (15 to 40 %)', 'abstract': '', 'values': [69],
                     'color': '#5dc39b', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Open (15 to 40 %) Water > 3 months (semi-) permenant',
                     'abstract': '', 'values': [70], 'color': '#5dc39b', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Woody Open (15 to 40 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [71], 'color': '#5dc39b', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Sparse (4 to 15 %)', 'abstract': '', 'values': [72],
                     'color': '#87ceb2', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Sparse (4 to 15 %) Water > 3 months (semi-) permenant',
                     'abstract': '', 'values': [73], 'color': '#87ceb2', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Woody Sparse (4 to 15 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [74], 'color': '#87ceb2', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Woody Scattered (1 to 4 %)', 'abstract': '', 'values': [75],
                     'color': '#b0dac9', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Woody Scattered (1 to 4 %) Water > 3 months (semi-) permenant',
                        'abstract': '', 'values': [76], 'color': '#b0dac9', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Woody Scattered (1 to 4 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [77], 'color': '#b0dac9', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Herbaceous Closed (> 65 %)', 'abstract': '', 'values': [78],
                     'color': '#27cc8b', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Closed (> 65 %) Water > 3 months (semi-) permenant',
                        'abstract': '', 'values': [79], 'color': '#27cc8b', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Closed (> 65 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [80], 'color': '#27cc8b', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Herbaceous Open (40 to 65 %)', 'abstract': '', 'values': [81],
                     'color': '#42d89f', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Open (40 to 65 %) Water > 3 months (semi-) permenant',
                        'abstract': '', 'values': [82], 'color': '#42d89f', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Open (40 to 65 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [83], 'color': '#42d89f', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Herbaceous Open (15 to 40 %)', 'abstract': '', 'values': [84],
                     'color': '#63e3b4', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Open (15 to 40 %) Water > 3 months (semi-) permenant',
                        'abstract': '', 'values': [85], 'color': '#63e3b4', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Open (15 to 40 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [86], 'color': '#63e3b4', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Herbaceous Sparse (4 to 15 %)', 'abstract': '',
                     'values': [87], 'color': '#87efc9', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Sparse (4 to 15 %) Water > 3 months (semi-) permenant',
                        'abstract': '', 'values': [88], 'color': '#87efc9', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Sparse (4 to 15 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [89], 'color': '#87efc9', 'alpha': 1},
                    {'title': 'Natural Aquatic Vegetated: Herbaceous Scattered (1 to 4 %)', 'abstract': '',
                     'values': [90], 'color': '#abfadd', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Scattered (1 to 4 %) Water > 3 months (semi-) permenant',
                        'abstract': '', 'values': [91], 'color': '#abfadd', 'alpha': 1},
                    {
                        'title': 'Natural Aquatic Vegetated: Herbaceous Scattered (1 to 4 %) Water < 3 months (temporary or seasonal)',
                        'abstract': '', 'values': [92], 'color': '#abfadd', 'alpha': 1},
                    {'title': 'Artificial Surface', 'abstract': '', 'values': [93], 'color': '#da5c69', 'alpha': 1},
                    {'title': '', 'abstract': '', 'values': [94], 'color': '#f3ab69', 'alpha': 1},
                    {'title': 'Natural Surface: Sparsely vegetated', 'abstract': '', 'values': [95], 'color': '#ffe68c',
                     'alpha': 1},
                    {'title': 'Natural Surface: Very sparsely vegetated', 'abstract': '', 'values': [96],
                     'color': '#fad26e', 'alpha': 1},
                    {'title': 'Natural Surface: Bare areas, unvegetated', 'abstract': '', 'values': [97],
                     'color': '#f3ab69', 'alpha': 1},
                    {'title': 'Water', 'abstract': '', 'values': [98], 'color': '#4d9fdc', 'alpha': 1},
                    {'title': '', 'abstract': '', 'values': [99], 'color': '#4d9fdc', 'alpha': 1},
                    {'title': 'Water: Tidal area', 'abstract': '', 'values': [100], 'color': '#bbdce9', 'alpha': 1},
                    {'title': 'Water: Perennial (> 9 months)', 'abstract': '', 'values': [101], 'color': '#1b55ba',
                     'alpha': 1},
                    {'title': 'Water: Non-perennial (7 to 9 months)', 'abstract': '', 'values': [102],
                     'color': '#3479c9', 'alpha': 1},
                    {'title': 'Water: Non-perennial (4 to 6 months)', 'abstract': '', 'values': [103],
                     'color': '#4f9dd9', 'alpha': 1},
                    {'title': 'Water: Non-perennial (1 to 3 months)', 'abstract': '', 'values': [104],
                     'color': '#85cafd', 'alpha': 1},
                    # {'title': 'Water: (Snow)', 'abstract': '', 'values': [105], 'color': '#fafafa', 'alpha': 1},

                ]
            }
        },
        {
            "animate": True,
            "preserve_user_date_order": True,
            "allowed_count_range": [3, 12],
            "frame_duration": 1000,
        }
    ],
}


layers = {
    "title": "DEA Land Cover",
    "name": "",
    "layers": [
        {
            "title": "DEA Land Cover Calendar Year (Landsat)",
            "name": "ga_ls_landcover",
            "abstract": """
            Land cover is the observed physical cover on the Earth's surface including trees, shrubs, grasses, soils, exposed rocks, water bodies, plantations, crops and built structures. A consistent, Australia-wide land cover product helps the understanding of how the different parts of the environment change and inter-relate. Earth observation data recorded over a period of time allows the observation of the state of land cover at specific times and therefore the way that land cover changes.
        """,
            "product_name": "ga_ls_landcover_class_cyear_2",
            "bands": bands_landcover,
            "time_resolution": "year",
            "resource_limits": reslim_wms_min_zoom_15_cache_rules,
            "dynamic": True,
            "native_crs": "EPSG:3577",
            "native_resolution": [25, -25],
            "image_processing": {
                "extent_mask_func": [],
                "always_fetch_bands": [],
                "manual_merge": False,
            },
            # "flags": [
            #     {
            #         "band": "land",
            #         "product": "geodata_coast_100k",
            #         "ignore_time": True,
            #         "ignore_info_flags": [],
            #     }
            # ],
            "styling": {
                "default_style": "level4",
                "styles": [style_c2_level4, style_c2_level3, style_c2_level3_test]
           },
            "feature_info": {
                "include_custom": {
                    "description": {
                        "function": f"{cfgbase}.lccs_local_cfg.lc_class_desc.class_labels",
                    }
                }
            },
        },
        {
            "title": "DEA Land Cover Environmental Descriptors",
            "name": "ga_ls_landcover_descriptors",
            "abstract": """
            Land cover is the observed physical cover on the Earth's surface including trees, shrubs, grasses, soils, exposed rocks, water bodies, plantations, crops and built structures. A consistent, Australia-wide land cover product helps the understanding of how the different parts of the environment change and inter-relate. Earth observation data recorded over a period of time allows the observation of the state of land cover at specific times and therefore the way that land cover changes.
        """,
            "product_name": "ga_ls_landcover_class_cyear_2",
            "bands": bands_eds,
            "time_resolution": "year",
            "resource_limits": reslim_wms_min_zoom_15_cache_rules,
            "dynamic": True,
            "native_crs": "EPSG:3577",
            "native_resolution": [25, -25],
            "image_processing": {
                "extent_mask_func": [],
                "always_fetch_bands": [],
                "manual_merge": False,
            },
            # "flags": [
            #     {
            #         "band": "land",
            #         "product": "geodata_coast_100k",
            #         "ignore_time": True,
            #         "ignore_info_flags": [],
            #     }
            # ],
            "styling": {
                "default_style": "canopyco",
                "styles": [style_c2_level4_canopyco, style_c2_level4_lifeform, style_c2_level4_watersea, style_c2_level4_waterper, style_c2_level4_baregrad],
            },
            "feature_info": {
                "include_custom": {
                    "description": {
                        "function": f"{cfgbase}.lccs_local_cfg.lc_class_desc.env_descriptor_labels",
                    }
                }
            }
        }
    ]
}


# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0

import sys
import traceback
from time import monotonic

from flask import g, render_template, request

from datacube_ows import __version__
from datacube_ows.http_utils import (
    capture_headers,
    get_service_base_url,
    lower_get_args,
    resp_headers,
)
from datacube_ows.index.api import ows_index
from datacube_ows.legend_generator import create_legend_for_style
from datacube_ows.ogc_exceptions import OGCException, WMSException
from datacube_ows.ows_configuration import get_config
from datacube_ows.protocol_versions import supported_versions
from datacube_ows.startup_utils import (
    initialise_aws_credentials,
    initialise_babel,
    initialise_debugging,
    initialise_flask,
    initialise_ignorable_warnings,
    initialise_logger,
    initialise_prometheus,
    initialise_sentry,
    parse_config_file,
    proxy_fix,
)
from datacube_ows.wcs1 import WCS_REQUESTS
from datacube_ows.wms import WMS_REQUESTS

# Logging intialisation
_LOG = initialise_logger()
initialise_ignorable_warnings()

# Initialisation of external libraries - controlled by environment variables.
initialise_debugging(_LOG)
initialise_sentry(_LOG)
initialise_aws_credentials(_LOG)

# Prepare parsed configuration object
cfg = parse_config_file()

# Initialise Flask
app = initialise_flask(__name__)

babel = initialise_babel(cfg, app)

# Initialisation of external libraries that depend on Flask
# (controlled by environment variables)
metrics = initialise_prometheus(app, _LOG)

# Add middleware to fix proxy headers, controlled by environment variables
app = proxy_fix(app, _LOG)

# Protocol/Version lookup table
OWS_SUPPORTED = supported_versions()

# Prometheus Metrics
prometheus_ows_ogc_metric = metrics.histogram(
    "ows_ogc",
    "Summary by OGC request protocol, version, operation, layer, and HTTP Status",
    labels={
        'query_request': lambda: request.args.get('request', "NONE").upper(),
        'query_service': lambda: request.args.get('service', "NONE").upper(),
        'query_version': lambda: request.args.get('version'),
        'query_layer': lambda: (request.args.get('query_layers') # WMS GetFeatureInfo
                                or request.args.get('layers')  # WMS
                                or request.args.get('layer')  # WMTS
                                or request.args.get('coverage')  # WCS 1.x
                                or request.args.get('coverageid')  # WCS 2.x
                                ),
        'status': lambda r: r.status_code,
    }
)


# Flask Routes


@app.route('/')
@prometheus_ows_ogc_metric
def ogc_impl():
    #pylint: disable=too-many-branches
    nocase_args = lower_get_args()
    nocase_args = capture_headers(request, nocase_args)
    service = nocase_args.get("service", "").upper()

    if service:
        return ogc_svc_impl(service.lower())

    # create dummy env if not exists
    try:
        # service argument is only required (in fact only defined) by OGC for
        # GetCapabilities requests.  As long as we are persisting with a single
        # routing end point for all services, we must derive the service from the request
        # parameter.
        # This is a quick hack to fix #64.  Service and operation routing could be
        # handled more elegantly.
        op = nocase_args.get("request", "").upper()
        if op in WMS_REQUESTS:
            return ogc_svc_impl("wms")
        elif op in WCS_REQUESTS:
            return ogc_svc_impl("wcs")
        elif op:
            # Should we return a WMS or WCS exception if there is no service specified?
            # Defaulting to WMS because that's what we already have.
            raise WMSException(
                "Invalid service and/or request",
                locator="Service and request parameters")
        else:
            cfg = get_config()   # pylint: disable=redefined-outer-name
            url = nocase_args.get('Host', nocase_args['url_root'])
            base_url = get_service_base_url(cfg.allowed_urls, url)
            return (render_template(
                            "index.html",
                            cfg=cfg,
                            supported=OWS_SUPPORTED,
                            base_url=base_url,
                            version=__version__,
                    ),
                    200,
                    resp_headers({"Content-Type": "text/html"}))
    except OGCException as e:
        _LOG.error("Handled Error: %s", repr(e.errors))
        return e.exception_response()
    except Exception as e: # pylint: disable=broad-except
        tb = sys.exc_info()[2]
        ogc_e = WMSException("Unexpected server error: %s" % str(e), http_response=500)
        return ogc_e.exception_response(traceback=traceback.extract_tb(tb))


def ogc_svc_impl(svc):
    svc_support = OWS_SUPPORTED.get(svc)
    nocase_args = lower_get_args()
    nocase_args = capture_headers(request, nocase_args)
    service = nocase_args.get("service", svc).upper()

    # Is service activated in config?
    try:
        if not svc_support:
            raise WMSException(f"Invalid service: {svc}",
                               valid_keys=[
                                       service.service
                                       for service in OWS_SUPPORTED.values()
                                       if service.activated()
                               ],
                               code=WMSException.OPERATION_NOT_SUPPORTED,
                               locator="service parameter")
        if not svc_support.activated():
            raise svc_support.default_exception_class("Invalid service and/or request", locator="Service and request parameters")

        # Does service match path (if supplied)
        if service != svc_support.service_upper:
            raise svc_support.default_exception_class("Invalid service", locator="Service parameter")

        version = nocase_args.get("version")
        version_support = svc_support.negotiated_version(version)
    except OGCException as e:
        return e.exception_response()

    try:
        return version_support.router(nocase_args)
    except OGCException as e:
        return e.exception_response()
    except Exception as e: #pylint: disable=broad-except
        tb = sys.exc_info()[2]
        ogc_e = version_support.exception_class("Unexpected server error: %s" % str(e), http_response=500)
        return ogc_e.exception_response(traceback=traceback.extract_tb(tb))


@app.route('/wms')
@prometheus_ows_ogc_metric
def ogc_wms_impl():
    return ogc_svc_impl("wms")


@app.route('/wmts')
@prometheus_ows_ogc_metric
def ogc_wmts_impl():
    return ogc_svc_impl("wmts")


@app.route('/wcs')
@prometheus_ows_ogc_metric
def ogc_wcs_impl():
    return ogc_svc_impl("wcs")

@app.route('/ping')
@metrics.summary('ows_heartbeat_pings', "Ping durations", labels={"status": lambda r: r.status})
def ping():
    dbs_ok = {
        name: ows_index(dc).check_db_access(dc)
        for name, dc in get_config().all_dcs.items()
    }

    if all(dbs_ok.values()):
        return render_template("ping.html", status="Up", statuses=dbs_ok), 200, resp_headers({"Content-Type": "text/html"})
    elif any(dbs_ok.values()):
        return render_template("ping.html", status="Partially Up", statuses=dbs_ok), 503, resp_headers({"Content-Type": "text/html"})
    else:
        return render_template("ping.html", status="Down", statuses=dbs_ok), 503, resp_headers({"Content-Type": "text/html"})


@app.route("/legend/<string:layer>/<string:style>/legend.png")
@metrics.histogram('ows_legends', "Legend query durations", labels={
    "layer": lambda: request.path.split("/")[2],
    "style": lambda: request.path.split("/")[3],
    "status": lambda r: r.status,
})
def legend(layer, style, dates=None):
    # pylint: disable=redefined-outer-name
    cfg = get_config()
    product = cfg.layer_index.get(layer)
    if not product:
        return "Unknown Layer", 404, resp_headers({"Content-Type": "text/plain"})
    if dates is None:
        args = lower_get_args()
        ndates = int(args.get("ndates", 0))
    else:
        ndates = len(dates)
    try:
        img = create_legend_for_style(product, style, ndates)
    except WMSException as e:
        return str(e), e.http_response, resp_headers({"Content-Type": "text/plain"})

    if not img:
        return "Unknown Style", 404, resp_headers({"Content-Type": "text/plain"})
    return img

# Flask middleware

@app.before_request
def start_timer():
    # pylint: disable=assigning-non-slot
    g.ogc_start_time = monotonic()


@app.after_request
def log_time_and_request_response(response):
    time_taken = int((monotonic() - g.ogc_start_time) * 1000)
    # request.environ.get('HTTP_X_REAL_IP') captures requester ip on a local docker container via gunicorn
    if request.environ.get('HTTP_X_REAL_IP'):
        ip = request.environ.get('HTTP_X_REAL_IP')
    # request.environ.get('HTTP_X_FORWARDED_FOR') captures request IP forwarded by ingress/loadbalancer
    elif request.environ.get('HTTP_X_FORWARDED_FOR'):
        ip = request.environ.get('HTTP_X_FORWARDED_FOR')
    # request.environ.get('REMOTE_ADDR') is standard internal IP address
    elif request.environ.get('REMOTE_ADDR'):
        ip = request.environ.get('REMOTE_ADDR')
    else:
        ip = 'Not found'
    _LOG.info("ip: %s request: %s returned status: %d and took: %d ms", ip, request.url, response.status_code, time_taken)
    return response



# This file is part of datacube-ows, part of the Open Data Cube project.
# See https://opendatacube.org for more information.
#
# Copyright (c) 2017-2024 OWS Contributors
# SPDX-License-Identifier: Apache-2.0




bands_c3_ls_common = {
    "nbart_blue": ["nbart_blue"],
    "nbart_green": ["nbart_green"],
    "nbart_red": ["nbart_red"],
    "nbart_nir": ["nbart_nir", "nbart_near_infrared"],
    "nbart_swir_1": ["nbart_swir_1", "nbart_shortwave_infrared_1"],
    "nbart_swir_2": ["nbart_swir_2", "nbart_shortwave_infrared_2"],
    "oa_nbart_contiguity": ["oa_nbart_contiguity", "nbart_contiguity"],
    "oa_fmask": ["oa_fmask", "fmask"],
}

bands_c3_ls_7 = bands_c3_ls_common.copy()
bands_c3_ls_7.update({
    "nbart_panchromatic": [],
})


bands_c3_ls_8 = bands_c3_ls_7.copy()
bands_c3_ls_8.update({
    "nbart_coastal_aerosol": ["coastal_aerosol",  "nbart_coastal_aerosol"],
})


bands_wofs_filt_sum = {"confidence": [], "wofs_filtered_summary": []}

bands_wofs_sum = {
    "count_wet": [],
    "count_clear": [],
    "frequency": [],
}

bands_wofs_obs = {
    "water": [],
}

bands_s2_ls_combo = {
    "nbart_red": ["nbart_red", "red"],
    "nbart_green": ["nbart_green", "green"],
    "nbart_blue": ["nbart_blue", "blue"],
    "nbart_common_nir": ["nbart_common_nir", "nir"],
    "nbart_common_swir_1": ["nbart_common_swir_1", "swir_1"],
    "nbart_common_swir_2": ["nbart_common_swir_2", "swir_2"],
    "oa_fmask": ["oa_fmask"],
}
